@@ -26,6 +26,7 @@ extern "C" {
 
 
 #include <sys/types.h>
+#include <stdint.h>
 
 #ifdef _WIN32
 typedef unsigned int size_t;
@@ -99,15 +100,14 @@ enum http_parser_type { HTTP_REQUEST, HTTP_RESPONSE, HTTP_BOTH };
 
 struct http_parser {
   /** PRIVATE **/
-  unsigned char type;
+  unsigned char type : 2;
+  unsigned char flags : 6;
   unsigned char state;
   unsigned char header_state;
   unsigned char index;
 
-  char flags;
-
-  size_t nread;
-  ssize_t content_length;
+  uint32_t nread;
+  int64_t content_length;
 
   /** READ-ONLY **/
   unsigned short http_major;
