@@ -23,7 +23,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 // UTILITY
-var inherits = require('sys').inherits;
+var util = require('util');
 var pSlice = Array.prototype.slice;
 
 // 1. The assert module provides functions that throw
@@ -47,7 +47,7 @@ assert.AssertionError = function AssertionError (options) {
     Error.captureStackTrace(this, stackStartFunction);
   }
 };
-inherits(assert.AssertionError, Error);
+util.inherits(assert.AssertionError, Error);
 
 assert.AssertionError.prototype.toString = function() {
   if (this.message) {
@@ -59,7 +59,7 @@ assert.AssertionError.prototype.toString = function() {
            , JSON.stringify(this.actual)
            ].join(" ");
   }
-}
+};
 
 // assert.AssertionError instanceof Error
 
@@ -131,6 +131,15 @@ function _deepEqual(actual, expected) {
   if (actual === expected) {
     return true;
 
+  } else if (Buffer.isBuffer(actual) && Buffer.isBuffer(expected)) {
+    if (actual.length != expected.length) return false;
+
+    for (var i = 0; i < actual.length; i++) {
+      if (actual[i] !== expected[i]) return false;
+    }
+
+    return true;
+
   // 7.2. If the expected value is a Date object, the actual value is
   // equivalent if it is also a Date object that refers to the same time.
   } else if (actual instanceof Date && expected instanceof Date) {
@@ -270,7 +279,7 @@ function _throws (shouldThrow, block, err, message) {
       (!shouldThrow && threw)) {
     throw exception;
   }
-}
+};
 
 // 11. Expected to throw an error:
 // assert.throws(block, Error_opt, message_opt);
