@@ -1,12 +1,13 @@
-var inherits = require('sys').inherits;
+var util = require('util');
 var EventEmitter = require('events').EventEmitter;
 var Stream = require('net').Stream;
 var InternalChildProcess = process.binding('child_process').ChildProcess;
+var constants;
 
 
-var spawn = exports.spawn = function (path, args, env, customFds) {
+var spawn = exports.spawn = function (path, args /*, options OR env, customFds */) {
   var child = new ChildProcess();
-  child.spawn(path, args, env, customFds);
+  child.spawn.apply(child, arguments);
   return child;
 };
 
@@ -27,7 +28,8 @@ exports.execFile = function (file /* args, options, callback */) {
   var options = { encoding: 'utf8'
                 , timeout: 0
                 , maxBuffer: 200*1024
-                , killSignal: 'SIGKILL'
+                , killSignal: 'SIGTERM'
+                , cwd: null
                 , env: null
                 };
   var args, optionArg, callback;
@@ -49,25 +51,53 @@ exports.execFile = function (file /* args, options, callback */) {
   // Merge optionArg into options
   if (optionArg) {
     var keys = Object.keys(options);
-    for (var i = 0; i < keys.length; i++) {
+    for (var i = 0, len = keys.length; i < len; i++) {
       var k = keys[i];
       if (optionArg[k] !== undefined) options[k] = optionArg[k];
     }
   }
 
-  var child = spawn(file, args, options.env);
+  var child = spawn(file, args, {cwd: options.cwd, env: options.env});
   var stdout = "";
   var stderr = "";
   var killed = false;
-
+  var exited = false;
   var timeoutId;
+
+  function exithandler (code, signal) {
+    if (exited) return;
+    exited = true;
+
+    if (timeoutId) {
+      clearTimeout(timeoutId);
+      timeoutId = null;
+    }
+
+    if (!callback) return;
+
+    if (code === 0 && signal === null) {
+      callback(null, stdout, stderr);
+    } else {
+      var e = new Error("Command failed: " + stderr);
+      e.killed = child.killed || killed;
+      e.code = code;
+      e.signal = signal;
+      callback(e, stdout, stderr);
+    }
+  }
+
+  function kill () {
+    killed = true;
+    child.kill(options.killSignal);
+    process.nextTick(function () {
+      exithandler(null, options.killSignal)
+    });
+  }
+
   if (options.timeout > 0) {
     timeoutId = setTimeout(function () {
-      if (!killed) {
-        child.kill(options.killSignal);
-        killed = true;
-        timeoutId = null;
-      }
+      kill();
+      timeoutId = null;
     }, options.timeout);
   }
 
@@ -76,32 +106,19 @@ exports.execFile = function (file /* args, options, callback */) {
 
   child.stdout.addListener("data", function (chunk) {
     stdout += chunk;
-    if (!killed && stdout.length > options.maxBuffer) {
-      child.kill(options.killSignal);
-      killed = true;
+    if (stdout.length > options.maxBuffer) {
+      kill();
     }
   });
 
   child.stderr.addListener("data", function (chunk) {
     stderr += chunk;
-    if (!killed && stderr.length > options.maxBuffer) {
-      child.kill(options.killSignal);
-      killed = true
+    if (stderr.length > options.maxBuffer) {
+      kill();
     }
   });
 
-  child.addListener("exit", function (code, signal) {
-    if (timeoutId) clearTimeout(timeoutId);
-    if (code === 0 && signal === null) {
-      if (callback) callback(null, stdout, stderr);
-    } else {
-      var e = new Error("Command failed: " + stderr);
-      e.killed = killed;
-      e.code = code;
-      e.signal = signal;
-      if (callback) callback(e, stdout, stderr);
-    }
-  });
+  child.addListener("exit", exithandler);
 
   return child;
 };
@@ -112,6 +129,8 @@ function ChildProcess () {
 
   var self = this;
 
+  this.killed = false;
+
   var gotCHLD = false;
   var exitCode;
   var termSignal;
@@ -153,17 +172,38 @@ function ChildProcess () {
 
   this.__defineGetter__('pid', function () { return internal.pid; });
 }
-inherits(ChildProcess, EventEmitter);
+util.inherits(ChildProcess, EventEmitter);
 
 
 ChildProcess.prototype.kill = function (sig) {
-  return this._internal.kill(sig);
+  if (this._internal.pid) {
+    this.killed = true;
+    if (!constants) constants = process.binding("constants");
+    sig = sig || 'SIGTERM';
+    if (!constants[sig]) throw new Error("Unknown signal: " + sig);
+    return this._internal.kill(constants[sig]);
+  }
 };
 
 
-ChildProcess.prototype.spawn = function (path, args, env, customFds) {
+ChildProcess.prototype.spawn = function (path, args, options, customFds) {
   args = args || [];
-  env = env || process.env;
+
+  var cwd, env;
+  if (!options || options.cwd === undefined &&
+      options.env === undefined &&
+      options.customFds === undefined) {
+    // Deprecated API: (path, args, options, env, customFds)
+    cwd = "";
+    env = options || process.env;
+    customFds = customFds || [-1, -1, -1];
+  } else {
+    // Recommended API: (path, args, options)
+    cwd = options.cwd || "";
+    env = options.env || process.env;
+    customFds = options.customFds || [-1, -1, -1];
+  }
+
   var envPairs = [];
   var keys = Object.keys(env);
   for (var index = 0, keysLength = keys.length; index < keysLength; index++) {
@@ -171,15 +211,13 @@ ChildProcess.prototype.spawn = function (path, args, env, customFds) {
     envPairs.push(key + "=" + env[key]);
   }
 
-  customFds = customFds || [-1, -1, -1];
-  var fds = this.fds = this._internal.spawn(path, args, envPairs, customFds);
+  var fds = this.fds = this._internal.spawn(path, args, cwd, envPairs, customFds);
 
   if (customFds[0] === -1 || customFds[0] === undefined) {
     this.stdin.open(fds[0]);
     this.stdin.writable = true;
     this.stdin.readable = false;
-  }
-  else {
+  } else {
     this.stdin = null;
   }
 
@@ -188,8 +226,7 @@ ChildProcess.prototype.spawn = function (path, args, env, customFds) {
     this.stdout.writable = false;
     this.stdout.readable = true;
     this.stdout.resume();
-  }
-  else {
+  } else {
     this.stdout = null;
   }
 
@@ -198,8 +235,7 @@ ChildProcess.prototype.spawn = function (path, args, env, customFds) {
     this.stderr.writable = false;
     this.stderr.readable = true;
     this.stderr.resume();
-  }
-  else {
+  } else {
     this.stderr = null;
   }
 };
