@@ -1,10 +1,9 @@
-var sys = require("sys");
+var util = require("util");
 var fs = require("fs");
 var events = require("events");
 var dns = require('dns');
 
-var Buffer = require('buffer').Buffer;
-var IOWatcher    = process.IOWatcher;
+var IOWatcher    = process.binding('io_watcher').IOWatcher;
 var binding      = process.binding('net');
 var socket       = binding.socket;
 var recvfrom     = binding.recvfrom;
@@ -28,6 +27,16 @@ function getPool() {
   return pool;
 }
 
+function dnsLookup(type, hostname, callback) {
+  var family = (type ? ((type === "udp6") ? 6 : 4) : null);
+  dns.lookup(hostname, family, function (err, ip, addressFamily) {
+    if (!err && family && addressFamily !== family) {
+      err = new Error('no address found in family '+type+' for '+hostname);
+    }
+    callback(err, ip, addressFamily);
+  });
+}
+
 function Socket (type, listener) {
   events.EventEmitter.call(this);
   var self = this;
@@ -63,7 +72,7 @@ function Socket (type, listener) {
   }
 }
 
-sys.inherits(Socket, events.EventEmitter);
+util.inherits(Socket, events.EventEmitter);
 exports.Socket = Socket;
 
 exports.createSocket = function (type, listener) {
@@ -103,22 +112,15 @@ Socket.prototype.bind = function () {
     } else {
       // the first argument is the port, the second an address
       this.port = arguments[0];
-      if (dns.isIP(arguments[1])) {
-        this.address = arguments[1];
-        binding.bind(self.fd, port, arguments[1]);
-        this.emit("listening");
-      } else {
-        dns.lookup(arguments[1], function (err, ip, addressType) {
-          // TODO - only look up proper record for address family
-          if (err) {
-            self.emit('error', err);
-          } else {
-            self.ip = ip;
-            binding.bind(self.fd, self.port, ip);
-            self.emit("listening");
-          }
-        });
-      }
+      dnsLookup(this.type, arguments[1], function (err, ip, addressFamily) {
+        if (err) {
+          self.emit('error', err);
+        } else {
+          self.ip = ip;
+          binding.bind(self.fd, self.port, ip);
+          self.emit("listening");
+        }
+      });
     }
   }
 };
@@ -153,7 +155,7 @@ Socket.prototype.setTTL = function(arg) {
   }
 };
 
-// translate arguments from JS API into C++ API, after optional DNS lookup
+// translate arguments from JS API into C++ API, possibly after DNS lookup
 Socket.prototype.send = function(buffer, offset, length) {
   var self = this;
 
@@ -172,14 +174,13 @@ Socket.prototype.send = function(buffer, offset, length) {
       throw new Error(this.type + " sockets must send to port, address");
     }
     
-    if (dns.isIP(arguments[4])) {
-      self.sendto(arguments[0], arguments[1], arguments[2], arguments[3], arguments[4], arguments[5]);
+    if (binding.isIP(arguments[4])) {
+      self.sendto(arguments[0], arguments[1], arguments[2], arguments[3],
+                  arguments[4], arguments[5]);
     } else {
       var port = arguments[3],
           callback = arguments[5];
-      
-      // TODO - only look up proper record for address family
-      dns.lookup(arguments[4], function (err, ip, addressType) {
+      dnsLookup(this.type, arguments[4], function (err, ip, addressFamily) {
         if (err) {  // DNS error
           if (callback) {
             callback(err);
