@@ -1,40 +1,36 @@
 var dns = process.binding('cares');
+var net = process.binding('net');
+var IOWatcher = process.binding('io_watcher').IOWatcher;
 
 
 var watchers = {};
 var activeWatchers = {};
+var Timer = process.binding('timer').Timer;
 
-
-var timer = new process.Timer();
+var timer = new Timer();
 
 timer.callback = function () {
   var sockets = Object.keys(activeWatchers);
   for (var i = 0, l = sockets.length; i < l; i++) {
     var socket = sockets[i];
-    var s = parseInt(socket);
+    var s = parseInt(socket, 10);
     channel.processFD( watchers[socket].read  ? s : dns.SOCKET_BAD
                      , watchers[socket].write ? s : dns.SOCKET_BAD
                      );
   }
   updateTimer();
-}
+};
 
 
 function updateTimer() {
   timer.stop();
 
   // Were just checking to see if activeWatchers is empty or not
-  for (var socket in activeWatchers) {
-    if (activeWatchers.hasOwnProperty(socket)) {
-      var max = 20000;
-      var timeout = channel.timeout(max);
-
-      timer.start(timeout, 0);
-      // Short circuit the loop on first find.
-      return;
-    }
-  }
-}
+  if (0 === Object.keys(activeWatchers).length) return;
+  var max = 20000;
+  var timeout = channel.timeout(max);
+  timer.start(timeout, 0);
+};
 
 
 var channel = new dns.Channel({SOCK_STATE_CB: function (socket, read, write) {
@@ -43,7 +39,7 @@ var channel = new dns.Channel({SOCK_STATE_CB: function (socket, read, write) {
   if (socket in watchers) {
     watcher = watchers[socket].watcher;
   } else {
-    watcher = new process.IOWatcher();
+    watcher = new IOWatcher();
     watchers[socket] = { read: read
                        , write: write
                        , watcher: watcher
@@ -54,8 +50,8 @@ var channel = new dns.Channel({SOCK_STATE_CB: function (socket, read, write) {
                        , write ? socket : dns.SOCKET_BAD
                        );
       updateTimer();
-    }
-  }
+    };
+  };
 
   watcher.set(socket, read == 1, write == 1);
 
@@ -71,8 +67,6 @@ var channel = new dns.Channel({SOCK_STATE_CB: function (socket, read, write) {
   updateTimer();
 }});
 
-exports.isIP = dns.isIP;
-
 exports.resolve = function (domain, type_, callback_) {
   var type, callback;
   if (typeof(type_) == 'string') {
@@ -90,7 +84,7 @@ exports.resolve = function (domain, type_, callback_) {
   } else {
     throw new Error('Unknown type "' + type + '"');
   }
-}
+};
 
 
 function familyToSym(family) {
@@ -98,7 +92,7 @@ function familyToSym(family) {
     family = (family === 6) ? dns.AF_INET6 : dns.AF_INET;
   }
   return family;
-}
+};
 
 
 exports.getHostByName = function (domain, family/*=4*/, callback) {
@@ -120,7 +114,7 @@ exports.lookup = function (domain, family, callback) {
     callback = family;
     family = undefined;
   } else if (family && family !== 4 && family !== 6) {
-    family = parseInt(family);
+    family = parseInt(family, 10);
     if (family === dns.AF_INET) {
       family = 4;
     } else if (family === dns.AF_INET6) {
@@ -133,7 +127,7 @@ exports.lookup = function (domain, family, callback) {
     callback(null, null, family === 6 ? 6 : 4);
     return;
   }
-  var matchedFamily = dns.isIP(domain);
+  var matchedFamily = net.isIP(domain);
   if (matchedFamily) {
     callback(null, domain, matchedFamily);
   } else {
@@ -149,7 +143,7 @@ exports.lookup = function (domain, family, callback) {
         var af = familyToSym(family);
         channel.getHostByName(domain, af, function (err, domains) {
           if (!err && domains && domains.length) {
-            if (family !== dns.isIP(domains[0])) {
+            if (family !== net.isIP(domains[0])) {
               callback(new Error('not found'), []);
             } else {
               callback(null, domains[0], family);
@@ -209,7 +203,7 @@ var resolveMap = {
   'TXT' : exports.resolveTxt,
   'SRV' : exports.resolveSrv,
   'PTR' : exports.resolvePtr,
-  'NS'  : exports.resolveNs,
+  'NS'  : exports.resolveNs
 };
 
 // ERROR CODES
