@@ -9,7 +9,7 @@ EventEmitter.prototype.emit = function (type) {
         (isArray(this._events.error) && !this._events.error.length))
     {
       if (arguments[1] instanceof Error) {
-        throw arguments[1];
+        throw arguments[1]; // Unhandled 'error' event
       } else {
         throw new Error("Uncaught, unspecified 'error' event.");
       }
@@ -76,6 +76,14 @@ EventEmitter.prototype.addListener = function (type, listener) {
 
 EventEmitter.prototype.on = EventEmitter.prototype.addListener;
 
+EventEmitter.prototype.once = function (type, listener) {
+  var self = this;
+  self.on(type, function g () {
+    self.removeListener(type, g);
+    listener.apply(this, arguments);
+  });
+};
+
 EventEmitter.prototype.removeListener = function (type, listener) {
   if ('function' !== typeof listener) {
     throw new Error('removeListener only takes instances of Function');
@@ -113,9 +121,3 @@ EventEmitter.prototype.listeners = function (type) {
   }
   return this._events[type];
 };
-exports.Promise = function removed () {
-  throw new Error(
-    'Promise has been removed. See '+
-    'http://groups.google.com/group/nodejs/msg/0c483b891c56fea2 for more information.');
-}
-process.Promise = exports.Promise;
