@@ -1,112 +1,126 @@
-var sys = require('sys');
-var events = require('events');
-var Buffer = require('buffer').Buffer;
+var util = require('util');
 
 var binding = process.binding('fs');
+var constants = process.binding('constants');
 var fs = exports;
+var Stream = require('stream').Stream;
 
 var kMinPoolSpace = 128;
-var kPoolSize = 40*1024;
+var kPoolSize = 40 * 1024;
 
 fs.Stats = binding.Stats;
 
 fs.Stats.prototype._checkModeProperty = function (property) {
-  return ((this.mode & process.S_IFMT) === property);
+  return ((this.mode & constants.S_IFMT) === property);
 };
 
 fs.Stats.prototype.isDirectory = function () {
-  return this._checkModeProperty(process.S_IFDIR);
+  return this._checkModeProperty(constants.S_IFDIR);
 };
 
 fs.Stats.prototype.isFile = function () {
-  return this._checkModeProperty(process.S_IFREG);
+  return this._checkModeProperty(constants.S_IFREG);
 };
 
 fs.Stats.prototype.isBlockDevice = function () {
-  return this._checkModeProperty(process.S_IFBLK);
+  return this._checkModeProperty(constants.S_IFBLK);
 };
 
 fs.Stats.prototype.isCharacterDevice = function () {
-  return this._checkModeProperty(process.S_IFCHR);
+  return this._checkModeProperty(constants.S_IFCHR);
 };
 
 fs.Stats.prototype.isSymbolicLink = function () {
-  return this._checkModeProperty(process.S_IFLNK);
+  return this._checkModeProperty(constants.S_IFLNK);
 };
 
 fs.Stats.prototype.isFIFO = function () {
-  return this._checkModeProperty(process.S_IFIFO);
+  return this._checkModeProperty(constants.S_IFIFO);
 };
 
 fs.Stats.prototype.isSocket = function () {
-  return this._checkModeProperty(process.S_IFSOCK);
+  return this._checkModeProperty(constants.S_IFSOCK);
 };
 
-fs.readFile = function (path, encoding_, callback) {
-  var encoding = typeof(encoding_) == 'string' ? encoding_ : null;
-  var callback_ = arguments[arguments.length - 1];
-  var callback = (typeof(callback_) == 'function' ? callback_ : noop);
-  binding.stat(path, function (err, stat) {
-    if (err) { callback(err); return; }
-    binding.open(path, process.O_RDONLY, 0666, function (err, fd) {
-      if (err) { callback(err); return; }
-      var size = stat.size;
-      var buffer = new Buffer(size);
-      var offset = 0;
-      function doRead() {
-        if (size < 1) {
-          binding.close(fd);
-          callback(null, buffer);
-          return;
-        }
-        // position is offset or null so we can read files on unseekable mediums
-        binding.read(fd, buffer, offset, size - offset, offset || null, function (err, amount) {
-          if (err) {
-            callback(err);
-            binding.close(fd);
-            return;
-          }
-          if (amount + offset < size) {
-            offset += amount;
-            doRead();
-            return;
-          }
-          binding.close(fd);
-          if (encoding) {
-            try {
-              var str = buffer.toString(encoding);
-            } catch (err) {
-              callback(err);
-              return;
-            }
-            callback(null, str);
-          } else {
-            callback(null, buffer);
-          }
+fs.readFile = function (path, encoding_) {
+  var encoding = typeof(encoding_) === 'string' ? encoding_ : null;
+  var callback = arguments[arguments.length-1];
+  if (typeof(callback) !== 'function') callback = noop;
+  var readStream = fs.createReadStream(path);
+  var buffers = [];
+  var nread = 0;
+  readStream.on("data", function (chunk) {
+    buffers.push(chunk);
+    nread += chunk.length;
+  });
+  readStream.on("error", function (er) {
+    callback(er);
+    readStream.destroy();
+  });
+  readStream.on("end", function () {
+    // copy all the buffers into one
+    var buffer;
+    switch (buffers.length) {
+      case 0: buffer = new Buffer(0); break;
+      case 1: buffer = buffers[0]; break;
+      default: // concat together
+        buffer = new Buffer(nread);
+        var n = 0;
+        buffers.forEach(function (b) {
+          var l = b.length;
+          b.copy(buffer, n, 0, l);
+          n += l;
         });
+        break;
+    }
+    if (encoding) {
+      try {
+        buffer = buffer.toString(encoding);
+      } catch (er) {
+        return callback(er);
       }
-      doRead();
-    });
+    }
+    callback(null, buffer);
   });
 };
 
 fs.readFileSync = function (path, encoding) {
-  var fd = fs.openSync(path, process.O_RDONLY, 0666);
-  var stat = fs.statSync(path);
-  var buffer = new Buffer(stat.size);
+  var fd = fs.openSync(path, constants.O_RDONLY, 0666);
+  var buffer = new Buffer(4048);
+  var buffers = [];
   var nread = 0;
+  var lastRead = 0;
 
-  while (nread < buffer.length) {
-    nread += fs.readSync(fd, buffer, nread, buffer.length - nread, null);
-  }
+  do {
+    if (lastRead) {
+      buffer._bytesRead = lastRead;
+      nread += lastRead;
+      buffers.push(buffer);
+    }
+    var buffer = new Buffer(4048);
+    lastRead = fs.readSync(fd, buffer, 0, buffer.length, null);
+  } while (lastRead > 0);
 
   fs.closeSync(fd);
 
-  if (encoding) {
-    return buffer.toString(encoding);
+  if (buffers.length > 1) {
+    var offset = 0;
+    var i;
+    buffer = new Buffer(nread);
+    buffers.forEach(function (i) {
+      if (!i._bytesRead) return;
+      i.copy(buffer,offset,0,i._bytesRead);
+      offset += i._bytesRead;
+    });
+  } else if (buffers.length) {
+    //buffers has exactly 1 (possibly zero length) buffer, so this should be a shortcut
+    buffer = buffers[0].slice(0, buffers[0]._bytesRead);
   } else {
-    return buffer;
+    buffer = new Buffer(0);
   }
+
+  if (encoding) buffer = buffer.toString(encoding);
+  return buffer;
 };
 
 
@@ -117,12 +131,12 @@ function stringToFlags(flag) {
     return flag;
   }
   switch (flag) {
-    case "r": return process.O_RDONLY;
-    case "r+": return process.O_RDWR;
-    case "w": return process.O_CREAT | process.O_TRUNC | process.O_WRONLY;
-    case "w+": return process.O_CREAT | process.O_TRUNC | process.O_RDWR;
-    case "a": return process.O_APPEND | process.O_CREAT | process.O_WRONLY;
-    case "a+": return process.O_APPEND | process.O_CREAT | process.O_RDWR;
+    case "r": return constants.O_RDONLY;
+    case "r+": return constants.O_RDWR;
+    case "w": return constants.O_CREAT | constants.O_TRUNC | constants.O_WRONLY;
+    case "w+": return constants.O_CREAT | constants.O_TRUNC | constants.O_RDWR;
+    case "a": return constants.O_APPEND | constants.O_CREAT | constants.O_WRONLY;
+    case "a+": return constants.O_APPEND | constants.O_CREAT | constants.O_RDWR;
     default: throw new Error("Unknown file open flag: " + flag);
   }
 }
@@ -140,8 +154,11 @@ fs.closeSync = function (fd) {
   return binding.close(fd);
 };
 
-fs.open = function (path, flags, mode, callback) {
-  if (mode === undefined) { mode = 0666; }
+fs.open = function (path, flags, mode_, callback) {
+  var mode = (typeof(mode_) == 'number' ? mode_ : 0666);
+  var callback_ = arguments[arguments.length - 1];
+  var callback = (typeof(callback_) == 'function' ? callback_ : null);
+
   binding.open(path, stringToFlags(flags), mode, callback || noop);
 };
 
@@ -208,7 +225,15 @@ fs.write = function (fd, buffer, offset, length, position, callback) {
     offset = 0;
     length = buffer.length;
   }
-  if(!length) return;
+
+  if (!length) {
+    if (typeof callback == 'function') {
+      process.nextTick(function() {
+        callback(undefined, 0);
+      });
+    }
+    return;
+  }
 
   binding.write(fd, buffer, offset, length, position, callback || noop);
 };
@@ -404,15 +429,6 @@ fs.writeFileSync = function (path, data, encoding) {
   fs.closeSync(fd);
 };
 
-fs.cat = function () {
-  throw new Error("fs.cat is deprecated. Please use fs.readFile instead.");
-};
-
-
-fs.catSync = function () {
-  throw new Error("fs.catSync is deprecated. Please use fs.readFileSync instead.");
-};
-
 // Stat Change Watchers
 
 var statWatchers = {};
@@ -445,6 +461,7 @@ fs.watchFile = function (filename) {
 };
 
 fs.unwatchFile = function (filename) {
+  var stat;
   if (statWatchers[filename]) {
     stat = statWatchers[filename];
     stat.stop();
@@ -458,132 +475,121 @@ var path = require('path');
 var normalize = path.normalize;
 var normalizeArray = path.normalizeArray;
 
-fs.realpathSync = function (path) {
-  var seen_links = {}, knownHards = {}, buf, i = 0, part, x, stats;
-  if (path.charAt(0) !== '/') {
-    var cwd = process.cwd().split('/');
-    path = cwd.concat(path.split('/'));
-    path = normalizeArray(path);
-    i = cwd.length;
-    buf = [].concat(cwd);
-  } else {
-    path = normalizeArray(path.split('/'));
+// realpath
+// Not using realpath(2) because it's bad.
+// See: http://insanecoding.blogspot.com/2007/11/pathmax-simply-isnt.html
+fs.realpathSync = realpathSync;
+fs.realpath = realpath;
+function realpathSync (p) {
+  if (p.charAt(0) !== '/') {
+    p = path.join(process.cwd(), p);
+  }
+  p = p.split('/');
+  var buf = [ '' ];
+  var seenLinks = {};
+  var knownHard = {};
+  // walk down the path, swapping out linked pathparts for their real
+  // values, and pushing non-link path bits onto the buffer.
+  // then return the buffer.
+  // NB: path.length changes.
+  for (var i = 0; i < p.length; i ++) {
+    // skip over empty path parts.
+    if (p[i] === '') continue;
+    var part = buf.join('/')+'/'+p[i];
+    if (knownHard[part]) {
+      buf.push( p[i] );
+      continue;
+    }
+    var stat = fs.lstatSync(part);
+    if (!stat.isSymbolicLink()) {
+      // not a symlink. easy.
+      knownHard[ part ] = true;
+      buf.push(p[i]);
+      continue;
+    }
+    var id = stat.dev.toString(32)+':'+stat.ino.toString(32);
+    if (seenLinks[id]) throw new Error("cyclic link at "+part);
+    seenLinks[id] = true;
+    var target = fs.readlinkSync(part);
+    if (target.charAt(0) === '/') {
+      // absolute. Start over.
+      buf = [''];
+      p = path.normalizeArray(target.split('/').concat(p.slice(i + 1)));
+      i = 0;
+      continue;
+    }
+    // not absolute.  join and splice.
+    target = target.split('/');
+    Array.prototype.splice.apply(p, [i, 1].concat(target));
+    p = path.normalizeArray(p);
+    i = 0;
     buf = [''];
   }
-  for (; i<path.length; i++) {
-    part = path.slice(0, i+1).join('/');
-    if (part.length !== 0) {
-      if (part in knownHards) {
-        buf.push(path[i]);
-      } else {
-        stats = fs.lstatSync(part);
-        if (stats.isSymbolicLink()) {
-          x = stats.dev.toString(32)+":"+stats.ino.toString(32);
-          if (x in seen_links)
-            throw new Error("cyclic link at "+part);
-          seen_links[x] = true;
-          part = fs.readlinkSync(part);
-          if (part.charAt(0) === '/') {
-            // absolute
-            path = normalizeArray(part.split('/'));
-            buf = [''];
-            i = 0;
-          } else {
-            // relative
-            Array.prototype.splice.apply(path, [i, 1].concat(part.split('/')));
-            part = normalizeArray(path);
-            var y = 0, L = Math.max(path.length, part.length), delta;
-            for (; y<L && path[y] === part[y]; y++);
-            if (y !== L) {
-              path = part;
-              delta = i-y;
-              i = y-1;
-              if (delta > 0) buf.splice(y, delta);
-            } else {
-              i--;
-            }
-          }
-        } else {
-          buf.push(path[i]);
-          knownHards[buf.join('/')] = true;
-        }
-      }
-    }
-  }
-  return buf.join('/');
+  return buf.join('/') || '/';
 }
 
 
-fs.realpath = function (path, callback) {
-  var seen_links = {}, knownHards = {}, buf = [''], i = 0, part, x;
-  if (path.charAt(0) !== '/') {
-    // assumes cwd is canonical
-    var cwd = process.cwd().split('/');
-    path = cwd.concat(path.split('/'));
-    path = normalizeArray(path);
-    i = cwd.length-1;
-    buf = [].concat(cwd);
-  } else {
-    path = normalizeArray(path.split('/'));
+function realpath (p, cb) {
+  if (p.charAt(0) !== '/') {
+    p = path.join(process.cwd(), p);
   }
-  function done(err) {
-    if (callback) {
-      if (!err) callback(err, buf.join('/'));
-      else callback(err);
+  p = p.split('/');
+  var buf = [ '' ];
+  var seenLinks = {};
+  var knownHard = {};
+  // walk down the path, swapping out linked pathparts for their real
+  // values, and pushing non-link path bits onto the buffer.
+  // then return the buffer.
+  // NB: path.length changes.
+  var i = 0;
+  var part;
+  LOOP();
+  function LOOP () {
+    i ++;
+    if (!(i < p.length)) return exit();
+    // skip over empty path parts.
+    if (p[i] === '') return process.nextTick(LOOP);
+    part = buf.join('/')+'/'+p[i];
+    if (knownHard[part]) {
+      buf.push( p[i] );
+      return process.nextTick(LOOP);
     }
+    return fs.lstat(part, gotStat);
   }
-  function next() {
-    if (++i === path.length) return done();
-    part = path.slice(0, i+1).join('/');
-    if (part.length === 0) return next();
-    if (part in knownHards) {
-      buf.push(path[i]);
-      next();
-    } else {
-      fs.lstat(part, function(err, stats){
-        if (err) return done(err);
-        if (stats.isSymbolicLink()) {
-          x = stats.dev.toString(32)+":"+stats.ino.toString(32);
-          if (x in seen_links)
-            return done(new Error("cyclic link at "+part));
-          seen_links[x] = true;
-          fs.readlink(part, function(err, npart){
-            if (err) return done(err);
-            part = npart;
-            if (part.charAt(0) === '/') {
-              // absolute
-              path = normalizeArray(part.split('/'));
-              buf = [''];
-              i = 0;
-            } else {
-              // relative
-              Array.prototype.splice.apply(path, [i, 1].concat(part.split('/')));
-              part = normalizeArray(path);
-              var y = 0, L = Math.max(path.length, part.length), delta;
-              for (; y<L && path[y] === part[y]; y++);
-              if (y !== L) {
-                path = part;
-                delta = i-y;
-                i = y-1; // resolve new node if needed
-                if (delta > 0) buf.splice(y, delta);
-              }
-              else {
-                i--; // resolve new node if needed
-              }
-            }
-            next();
-          }); // binding.readlink
-        }
-        else {
-          buf.push(path[i]);
-          knownHards[buf.join('/')] = true;
-          next();
-        }
-      }); // binding.lstat
+  function gotStat (er, stat) {
+    if (er) return cb(er);
+    if (!stat.isSymbolicLink()) {
+      // not a symlink. easy.
+      knownHard[ part ] = true;
+      buf.push(p[i]);
+      return process.nextTick(LOOP);
     }
+    var id = stat.dev.toString(32)+':'+stat.ino.toString(32);
+    if (seenLinks[id]) return cb(new Error("cyclic link at "+part));
+    seenLinks[id] = true;
+    fs.readlink(part, gotTarget);
   }
-  next();
-};
+  function gotTarget (er, target) {
+    if (er) return cb(er);
+    if (target.charAt(0) === '/') {
+      // absolute. Start over.
+      buf = [''];
+      p = path.normalizeArray(target.split('/').concat(p.slice(i + 1)));
+      i = 0;
+      return process.nextTick(LOOP);
+    }
+    // not absolute.  join and splice.
+    target = target.split('/');
+    Array.prototype.splice.apply(p, [i, 1].concat(target));
+    p = path.normalizeArray(p);
+    i = 0;
+    buf = [''];
+    return process.nextTick(LOOP);
+  }
+  function exit () {
+    cb(null, buf.join('/') || '/');
+  }
+}
 
 var pool;
 function allocNewPool () {
@@ -598,7 +604,11 @@ fs.createReadStream = function(path, options) {
 };
 
 var ReadStream = fs.ReadStream = function(path, options) {
-  events.EventEmitter.call(this);
+  if (!(this instanceof ReadStream)) return new ReadStream(path, options);
+
+  Stream.call(this);
+
+  var self = this;
 
   this.path = path;
   this.fd = null;
@@ -607,7 +617,7 @@ var ReadStream = fs.ReadStream = function(path, options) {
 
   this.flags = 'r';
   this.mode = 0666;
-  this.bufferSize = 4 * 1024;
+  this.bufferSize = 64 * 1024;
 
   options = options || {};
 
@@ -618,11 +628,14 @@ var ReadStream = fs.ReadStream = function(path, options) {
     this[key] = options[key];
   }
 
-  if(this.start || this.end) {
-    if(this.start === undefined || this.end === undefined) {
-      self.emit('error', new Error('Both start and end are needed for range streaming.'));
-    } else if(this.start > this.end) {
-      self.emit('error', new Error('start must be <= end'));
+  if(this.encoding) this.setEncoding(this.encoding);
+
+  if (this.start !== undefined || this.end !== undefined) {
+    if (this.start === undefined || this.end === undefined) {
+      this.emit('error',
+        new Error('Both start and end are needed for range streaming.'));
+    } else if (this.start > this.end) {
+      this.emit('error', new Error('start must be <= end'));
     } else {
       this.firstRead = true;
     }
@@ -632,7 +645,6 @@ var ReadStream = fs.ReadStream = function(path, options) {
     return;
   }
 
-  var self = this;
   fs.open(this.path, this.flags, this.mode, function(err, fd) {
     if (err) {
       self.emit('error', err);
@@ -645,7 +657,7 @@ var ReadStream = fs.ReadStream = function(path, options) {
     self._read();
   });
 };
-sys.inherits(ReadStream, events.EventEmitter);
+util.inherits(ReadStream, Stream);
 
 fs.FileReadStream = fs.ReadStream; // support the legacy name
 
@@ -666,9 +678,8 @@ ReadStream.prototype._read = function () {
     allocNewPool();
   }
 
-  if(this.start && this.firstRead) {
-    this.pos = this.start;
-    this.firstRead = false;
+  if (self.start !== undefined && self.firstRead) {
+    self.pos = self.start;
   }
 
   // Grab another reference to the pool in the case that while we're in the
@@ -678,7 +689,7 @@ ReadStream.prototype._read = function () {
   var toRead = Math.min(pool.length - pool.used, this.bufferSize);
   var start = pool.used;
 
-  if(this.pos) {
+  if (this.pos !== undefined) {
     toRead = Math.min(this.end - this.pos + 1, toRead);
   }
 
@@ -714,9 +725,12 @@ ReadStream.prototype._read = function () {
     self._read();
   }
 
-  fs.read(self.fd, pool, pool.used, toRead, this.pos, afterRead);
+  // pass null for position after we've seeked to the start of a range read
+  // always pass null on a non-range read
+  fs.read(self.fd, pool, pool.used, toRead, (self.firstRead ? self.pos : null), afterRead);
+  self.firstRead = false;
 
-  if(self.pos) {
+  if (self.pos !== undefined) {
     self.pos += toRead;
   }
   pool.used += toRead;
@@ -733,17 +747,6 @@ ReadStream.prototype._emitData = function (d) {
 };
 
 
-var readStreamForceCloseWarning;
-
-ReadStream.prototype.forceClose = function (cb) {
-  if (!readStreamForceCloseWarning) {
-    readStreamForceCloseWarning = "ReadStream.prototype.forceClose renamed to destroy()";
-    sys.error(readStreamForceCloseWarning);
-  }
-  return this.destroy(cb);
-};
-
-
 ReadStream.prototype.destroy = function (cb) {
   var self = this;
   this.readable = false;
@@ -792,11 +795,13 @@ fs.createWriteStream = function(path, options) {
 };
 
 var WriteStream = fs.WriteStream = function(path, options) {
-  events.EventEmitter.call(this);
+  if (!(this instanceof WriteStream)) return new WriteStream(path, options);
+
+  Stream.call(this);
 
   this.path = path;
   this.fd = null;
-  this.writeable = true;
+  this.writable = true;
 
   this.flags = 'w';
   this.encoding = 'binary';
@@ -819,17 +824,19 @@ var WriteStream = fs.WriteStream = function(path, options) {
     this.flush();
   }
 };
-sys.inherits(WriteStream, events.EventEmitter);
+util.inherits(WriteStream, Stream);
 
 fs.FileWriteStream = fs.WriteStream; // support the legacy name
 
-
 WriteStream.prototype.flush = function () {
   if (this.busy) return;
   var self = this;
 
   var args = this._queue.shift();
-  if (!args) return self.emit('drain');
+  if (!args) {
+    if (this.drainable) { self.emit('drain'); }
+    return;
+  }
 
   this.busy = true;
 
@@ -842,7 +849,7 @@ WriteStream.prototype.flush = function () {
     self.busy = false;
 
     if (err) {
-      self.writeable = false;
+      self.writable = false;
       if (cb) {
         cb(err);
       }
@@ -879,12 +886,13 @@ WriteStream.prototype.flush = function () {
   method.apply(this, args);
 };
 
-
 WriteStream.prototype.write = function (data) {
-  if (!this.writeable) {
-    throw new Error('stream not writeable');
+  if (!this.writable) {
+    throw new Error('stream not writable');
   }
 
+  this.drainable = true;
+
   var cb;
   if (typeof(arguments[arguments.length-1]) == 'function') {
     cb = arguments[arguments.length-1];
@@ -904,52 +912,33 @@ WriteStream.prototype.write = function (data) {
   return false;
 };
 
-
-var writeStreamCloseWarning;
-
-WriteStream.prototype.close = function (cb) {
-  if (!writeStreamCloseWarning) {
-    writeStreamCloseWarning = "WriteStream.prototype.close renamed to end()";
-    sys.error(writeStreamCloseWarning);
-  }
-  return this.end(cb);
-}
-
-
 WriteStream.prototype.end = function (cb) {
-  this.writeable = false;
+  this.writable = false;
   this._queue.push([fs.close, cb]);
   this.flush();
 };
 
+WriteStream.prototype.destroy = function (cb) {
+  var self = this;
+  this.writable = false;
 
-var writeStreamForceCloseWarning;
-
-WriteStream.prototype.forceClose = function (cb) {
-  if (!writeStreamForceCloseWarning) {
-    writeStreamForceCloseWarning = "WriteStream.prototype.forceClose renamed to destroy()";
-    sys.error(writeStreamForceCloseWarning);
-  }
-  return this.destroy(cb);
-};
-
-
-WriteStream.prototype.forceClose = function (cb) {
-  this.writeable = false;
-  fs.close(self.fd, function(err) {
-    if (err) {
-      if (cb) {
-        cb(err);
+  function close() {
+    fs.close(self.fd, function(err) {
+      if (err) {
+        if (cb) { cb(err); }
+        self.emit('error', err);
+        return;
       }
 
-      self.emit('error', err);
-      return;
-    }
+      if (cb) { cb(null); }
+      self.emit('close');
+    });
+  }
 
-    if (cb) {
-      cb(null);
-    }
-    self.emit('close');
-  });
+  if (this.fd) {
+    close();
+  } else {
+    this.addListener('open', close);
+  }
 };
 
