@@ -1,16 +1,15 @@
-var sys = require('sys');
+var util = require('util');
 
 var debug;
 var debugLevel = parseInt(process.env.NODE_DEBUG, 16);
 if (debugLevel & 0x4) {
-  debug = function (x) { sys.error('HTTP: ' + x); };
+  debug = function (x) { util.error('HTTP: ' + x); };
 } else {
   debug = function () { };
 }
 
 var net = require('net');
-var events = require('events');
-var Buffer = require('buffer').Buffer;
+var stream = require('stream');
 
 var FreeList = require('freelist').FreeList;
 var HTTPParser = process.binding('http_parser').HTTPParser;
@@ -61,6 +60,8 @@ var parsers = new FreeList('parsers', 1000, function () {
   parser.onHeadersComplete = function (info) {
     if (parser.field && (parser.value != undefined)) {
       parser.incoming._addHeaderLine(parser.field, parser.value);
+      parser.field = null;
+      parser.value = null;
     }
 
     parser.incoming.httpVersionMajor = info.versionMajor;
@@ -74,7 +75,7 @@ var parsers = new FreeList('parsers', 1000, function () {
       parser.incoming.method = info.method;
     } else {
       // client only
-      parser.incoming.statusCode = info.statusCode;
+      parser.incoming.statusCode = info.statusCode;  
     }
 
     parser.incoming.upgrade = info.upgrade;
@@ -102,6 +103,10 @@ var parsers = new FreeList('parsers', 1000, function () {
   };
 
   parser.onMessageComplete = function () {
+    this.incoming.complete = true;
+    if (parser.field && (parser.value != undefined)) {
+      parser.incoming._addHeaderLine(parser.field, parser.value);
+    }
     if (!parser.incoming.upgrade) {
       // For upgraded connections, also emit this after parser.execute
       parser.incoming.emit("end");
@@ -110,6 +115,7 @@ var parsers = new FreeList('parsers', 1000, function () {
 
   return parser;
 });
+exports.parsers = parsers;
 
 
 var CRLF = "\r\n";
@@ -173,18 +179,24 @@ var transferEncodingExpression = /Transfer-Encoding/i;
 var closeExpression = /close/i;
 var chunkExpression = /chunk/i;
 var contentLengthExpression = /Content-Length/i;
+var expectExpression = /Expect/i;
+var continueExpression = /100-continue/i;
 
 
 /* Abstract base class for ServerRequest and ClientResponse. */
 function IncomingMessage (socket) {
-  events.EventEmitter.call(this);
+  stream.Stream.call(this);
 
   // TODO Remove one of these eventually.
   this.socket = socket;
   this.connection = socket;
 
   this.httpVersion = null;
+  this.complete = false;
   this.headers = {};
+  this.trailers = {};
+
+  this.readable = true;
 
   // request (server) only
   this.url = "";
@@ -195,24 +207,14 @@ function IncomingMessage (socket) {
   this.statusCode = null;
   this.client = this.socket;
 }
-sys.inherits(IncomingMessage, events.EventEmitter);
+util.inherits(IncomingMessage, stream.Stream);
 exports.IncomingMessage = IncomingMessage;
 
-IncomingMessage.prototype._parseQueryString = function () {
-  throw new Error("_parseQueryString is deprecated. Use require(\"querystring\") to parse query strings.\n");
+
+IncomingMessage.prototype.destroy = function (error) {
+  this.socket.destroy(error);
 };
 
-var setBodyEncodingWarning;
-
-IncomingMessage.prototype.setBodyEncoding = function (enc) {
-  // deprecation message
-  if (!setBodyEncodingWarning) {
-    setBodyEncodingWarning = "setBodyEncoding has been renamed to setEncoding, please update your code.";
-    sys.error(setBodyEncodingWarning);
-  }
-
-  this.setEncoding(enc);
-};
 
 IncomingMessage.prototype.setEncoding = function (encoding) {
   var StringDecoder = require("string_decoder").StringDecoder; // lazy load
@@ -235,32 +237,55 @@ IncomingMessage.prototype.resume = function () {
 // and drop the second. Extended header fields (those beginning with 'x-') are
 // always joined.
 IncomingMessage.prototype._addHeaderLine = function (field, value) {
-  if (!(field in this.headers)) {
-    this.headers[field] = value;
-    return;
+  var dest;
+  if (this.complete) {
+      dest = this.trailers;
+  } else {
+      dest = this.headers;
   }
-
-  // If this field already exists in the request, use duplicate-resolution
-  // logic from RFC2616.
   switch (field) {
-  case 'accept':
-  case 'accept-charset':
-  case 'accept-encoding':
-  case 'accept-language':
-  case 'connection':
-  case 'cookie':
-    this.headers[field] += ', ' + value;
-    break;
+    // Array headers:
+    case 'set-cookie':
+      if (field in dest) {
+        dest[field].push(value);
+      } else {
+        dest[field] = [value];
+      }
+      break;
 
-  default:
-    if (field[0] !== 'x' || field[1] !== '-') break;
-    this.headers[field] += ', ' + value;
-    break;
+    // Comma separate. Maybe make these arrays?
+    case 'accept':
+    case 'accept-charset':
+    case 'accept-encoding':
+    case 'accept-language':
+    case 'connection':
+    case 'cookie':
+      if (field in dest) {
+        dest[field] += ', ' + value;
+      } else {
+        dest[field] = value;
+      }
+      break;
+
+
+    default:
+      if (field.slice(0,2) == 'x-') {
+        // except for x-
+        if (field in dest) {
+          dest[field] += ', ' + value;
+        } else {
+          dest[field] = value;
+        }
+      } else {
+        // drop duplicates
+        if (!(field in dest)) dest[field] = value;
+      }
+      break;
   }
 };
 
 function OutgoingMessage (socket) {
-  events.EventEmitter.call(this, socket);
+  stream.Stream.call(this);
 
   // TODO Remove one of these eventually.
   this.socket = socket;
@@ -269,20 +294,28 @@ function OutgoingMessage (socket) {
   this.output = [];
   this.outputEncodings = [];
 
+  this.writable = true;
+
   this._last = false;
   this.chunkedEncoding = false;
   this.shouldKeepAlive = true;
   this.useChunkedEncodingByDefault = true;
 
   this._hasBody = true;
+  this._trailer = '';
 
   this.finished = false;
 }
-sys.inherits(OutgoingMessage, events.EventEmitter);
+util.inherits(OutgoingMessage, stream.Stream);
 exports.OutgoingMessage = OutgoingMessage;
 
+
+OutgoingMessage.prototype.destroy = function (error) {
+  this.socket.destroy(error);
+};
+
+
 // This abstract either writing directly to the socket or buffering it.
-// Rename to _writeRaw() ?
 OutgoingMessage.prototype._send = function (data, encoding) {
   // This is a shameful hack to get the headers and first body chunk onto
   // the same packet. Future versions of Node are going to take care of
@@ -296,7 +329,10 @@ OutgoingMessage.prototype._send = function (data, encoding) {
     }
     this._headerSent = true;
   }
+  return this._writeRaw(data, encoding);
+};
 
+OutgoingMessage.prototype._writeRaw = function(data, encoding) {
   if (this.connection._outgoing[0] === this && this.connection.writable) {
     // There might be pending data in the this.output buffer.
     while (this.output.length) {
@@ -318,7 +354,6 @@ OutgoingMessage.prototype._send = function (data, encoding) {
 };
 
 OutgoingMessage.prototype._buffer = function (data, encoding) {
-  // Buffer
   if (data.length === 0) return;
 
   var length = this.output.length;
@@ -351,15 +386,42 @@ OutgoingMessage.prototype._storeHeader = function (firstLine, headers) {
   var sentConnectionHeader = false;
   var sentContentLengthHeader = false;
   var sentTransferEncodingHeader = false;
+  var sentExpect = false;
 
   // firstLine in the case of request is: "GET /index.html HTTP/1.1\r\n"
   // in the case of response it is: "HTTP/1.1 200 OK\r\n"
   var messageHeader = firstLine;
   var field, value;
+  var self = this;
+
+  function store(field, value) {
+    messageHeader += field + ": " + value + CRLF;
+
+    if (connectionExpression.test(field)) {
+      sentConnectionHeader = true;
+      if (closeExpression.test(value)) {
+        self._last = true;
+      } else {
+        self.shouldKeepAlive = true;
+      }
+
+    } else if (transferEncodingExpression.test(field)) {
+      sentTransferEncodingHeader = true;
+      if (chunkExpression.test(value)) self.chunkedEncoding = true;
+
+    } else if (contentLengthExpression.test(field)) {
+      sentContentLengthHeader = true;
+
+    } else if (expectExpression.test(field)) {
+      sentExpect = true;
+
+    }
+  }
 
   if (headers) {
     var keys = Object.keys(headers);
     var isArray = (Array.isArray(headers));
+
     for (var i = 0, l = keys.length; i < l; i++) {
       var key = keys[i];
       if (isArray) {
@@ -370,23 +432,12 @@ OutgoingMessage.prototype._storeHeader = function (firstLine, headers) {
         value = headers[key];
       }
 
-      messageHeader += field + ": " + value + CRLF;
-
-      if (connectionExpression.test(field)) {
-        sentConnectionHeader = true;
-        if (closeExpression.test(value)) {
-          this._last = true;
-        } else {
-          this.shouldKeepAlive = true;
+      if (Array.isArray(value)) {
+        for (var j = 0; j < value.length; j++) {
+          store(field, value[j]);
         }
-
-      } else if (transferEncodingExpression.test(field)) {
-        sentTransferEncodingHeader = true;
-        if (chunkExpression.test(value)) this.chunkedEncoding = true;
-
-      } else if (contentLengthExpression.test(field)) {
-        sentContentLengthHeader = true;
-
+      } else {
+        store(field, value);
       }
     }
   }
@@ -418,15 +469,13 @@ OutgoingMessage.prototype._storeHeader = function (firstLine, headers) {
 
   this._header = messageHeader + CRLF;
   this._headerSent = false;
-  // wait until the first body chunk, or close(), is sent to flush.
+  // wait until the first body chunk, or close(), is sent to flush,
+  // UNLESS we're sending Expect: 100-continue.
+  if (sentExpect) {
+      this._send("");
+  }
 };
 
-
-OutgoingMessage.prototype.sendBody = function () {
-  throw new Error("sendBody() has been renamed to write(). ");
-};
-
-
 OutgoingMessage.prototype.write = function (chunk, encoding) {
   if (!this._header) {
     throw new Error("You have to call writeHead() before write()");
@@ -450,7 +499,7 @@ OutgoingMessage.prototype.write = function (chunk, encoding) {
     if (typeof(chunk) === 'string') {
       len = Buffer.byteLength(chunk, encoding);
       var chunk = len.toString(16) + CRLF + chunk + CRLF;
-      debug('string chunk = ' + sys.inspect(chunk));
+      debug('string chunk = ' + util.inspect(chunk));
       ret = this._send(chunk, encoding);
     } else {
       // buffer
@@ -467,20 +516,26 @@ OutgoingMessage.prototype.write = function (chunk, encoding) {
   return ret;
 };
 
-OutgoingMessage.prototype.finish = function () {
-  throw new Error("finish() has been renamed to close().");
-};
 
-var closeWarning;
+OutgoingMessage.prototype.addTrailers = function (headers) {
+  this._trailer = "";
+  var keys = Object.keys(headers);
+  var isArray = (Array.isArray(headers));
+  for (var i = 0, l = keys.length; i < l; i++) {
+    var key = keys[i];
+    if (isArray) {
+      field = headers[key][0];
+      value = headers[key][1];
+    } else {
+      field = key;
+      value = headers[key];
+    }
 
-OutgoingMessage.prototype.close = function (data, encoding) {
-  if (!closeWarning) {
-    closeWarning = "OutgoingMessage.prototype.close has been renamed to end()";
-    sys.error(closeWarning);
+    this._trailer += field + ": " + value + CRLF;
   }
-  return this.end(data, encoding);
 };
 
+
 OutgoingMessage.prototype.end = function (data, encoding) {
   var ret;
 
@@ -503,7 +558,9 @@ OutgoingMessage.prototype.end = function (data, encoding) {
                                  + l
                                  + CRLF
                                  + data
-                                 + "\r\n0\r\n\r\n"
+                                 + "\r\n0\r\n"
+                                 + this._trailer
+                                 + "\r\n"
                                  , encoding
                                  );
     } else {
@@ -518,8 +575,8 @@ OutgoingMessage.prototype.end = function (data, encoding) {
 
   if (!hot) {
     if (this.chunkedEncoding) {
-      ret = this._send('0\r\n\r\n'); // Last chunk.
-    } else if (!data) {
+      ret = this._send('0\r\n' + this._trailer + '\r\n'); // Last chunk.
+    } else {
       // Force a flush, HACK.
       ret = this._send('');
     }
@@ -548,9 +605,13 @@ function ServerResponse (req) {
     this.shouldKeepAlive = false;
   }
 }
-sys.inherits(ServerResponse, OutgoingMessage);
+util.inherits(ServerResponse, OutgoingMessage);
 exports.ServerResponse = ServerResponse;
 
+ServerResponse.prototype.writeContinue = function () {
+     this._writeRaw("HTTP/1.1 100 Continue" + CRLF + CRLF, 'ascii');
+     this._sent100 = true;
+};
 
 ServerResponse.prototype.writeHead = function (statusCode) {
   var reasonPhrase, headers, headerIndex;
@@ -572,34 +633,34 @@ ServerResponse.prototype.writeHead = function (statusCode) {
   var statusLine = "HTTP/1.1 " + statusCode.toString() + " "
                   + reasonPhrase + CRLF;
 
-  if (statusCode === 204 || statusCode === 304) {
+  if (   statusCode === 204 
+      || statusCode === 304
+      || (statusCode >= 100 && statusCode <= 199)
+     ) {
     // RFC 2616, 10.2.5:
     // The 204 response MUST NOT include a message-body, and thus is always
     // terminated by the first empty line after the header fields.
     // RFC 2616, 10.3.5:
     // The 304 response MUST NOT contain a message-body, and thus is always
     // terminated by the first empty line after the header fields.
+    // RFC 2616, 10.1 Informational 1xx:
+    // This class of status code indicates a provisional response,
+    // consisting only of the Status-Line and optional headers, and is
+    // terminated by an empty line.
     this._hasBody = false;
   }
 
+  // don't keep alive connections where the client expects 100 Continue
+  // but we sent a final status; they may put extra bytes on the wire.
+  if (this._expect_continue && ! this._sent100) {
+      this.shouldKeepAlive = false;
+  }
 
   this._storeHeader(statusLine, headers);
 };
 
-// TODO Eventually remove
-var sendHeaderWarning, writeHeaderWarning;
-ServerResponse.prototype.sendHeader = function () {
-  if (!sendHeaderWarning) {
-    sendHeaderWarning = "sendHeader() has been renamed to writeHead()";
-    sys.error(sendHeaderWarning);
-  }
-  this.writeHead.apply(this, arguments);
-};
+
 ServerResponse.prototype.writeHeader = function () {
-  if (!writeHeaderWarning) {
-    writeHeaderWarning = "writeHeader() has been renamed to writeHead()";
-    sys.error(writeHeaderWarning);
-  }
   this.writeHead.apply(this, arguments);
 };
 
@@ -617,41 +678,9 @@ function ClientRequest (socket, method, url, headers) {
 
   this._storeHeader(method + " " + url + " HTTP/1.1\r\n", headers);
 }
-sys.inherits(ClientRequest, OutgoingMessage);
+util.inherits(ClientRequest, OutgoingMessage);
 exports.ClientRequest = ClientRequest;
 
-ClientRequest.prototype.finish = function () {
-  throw new Error( "finish() has been renamed to end() and no longer takes "
-                 + "a response handler as an argument. Manually add a 'response' listener "
-                 + "to the request object."
-                 );
-};
-
-var clientRequestCloseWarning;
-
-ClientRequest.prototype.close = function () {
-  if (!clientRequestCloseWarning) {
-    clientRequestCloseWarning = "Warning: ClientRequest.prototype.close has been renamed to end()";
-    sys.error(clientRequestCloseWarning);
-  }
-  if (typeof arguments[0] == "function") {
-    throw new Error( "ClientRequest.prototype.end does not take a callback. "
-                   + "Add a 'response' listener manually to the request object."
-                   );
-  }
-  return this.end();
-};
-
-ClientRequest.prototype.end = function () {
-  if (typeof arguments[0] == "function") {
-    throw new Error( "ClientRequest.prototype.end does not take a callback. "
-                   + "Add a 'response' listener manually to get the response."
-                   );
-  }
-  OutgoingMessage.prototype.end.apply(this, arguments);
-};
-
-
 function outgoingFlush (socket) {
   // This logic is probably a bit confusing. Let me explain a bit:
   //
@@ -713,6 +742,7 @@ function httpSocketSetup (socket) {
 
 
 function Server (requestListener) {
+  if (!(this instanceof Server)) return new Server(requestListener);
   net.Server.call(this);
 
   if(requestListener){
@@ -721,12 +751,12 @@ function Server (requestListener) {
 
   this.addListener("connection", connectionListener);
 }
-sys.inherits(Server, net.Server);
+util.inherits(Server, net.Server);
 
 Server.prototype.setSecure = function (credentials) {
   this.secure = true;
   this.credentials = credentials;
-}
+};
 
 exports.Server = Server;
 
@@ -827,67 +857,44 @@ function connectionListener (socket) {
     res.shouldKeepAlive = shouldKeepAlive;
     socket._outgoing.push(res);
 
-    self.emit('request', req, res);
+    if ('expect' in req.headers
+       && (req.httpVersionMajor == 1 && req.httpVersionMinor == 1)
+       && continueExpression.test(req.headers['expect'])) {
+        res._expect_continue = true;
+        if (self.listeners("checkContinue").length) {
+            self.emit("checkContinue", req, res);
+        } else {
+            res.writeContinue();
+            self.emit('request', req, res);
+        }
+    } else {
+        self.emit('request', req, res);
+    }
     return false; // Not a HEAD response. (Not even a response!)
   };
 }
 
 
 function Client ( ) {
+  if (!(this instanceof Client)) return new Client();
   net.Stream.call(this);
   var self = this;
 
   httpSocketSetup(self);
 
-  var parser;
-
-  function initParser () {
-    if (!parser) parser = parsers.alloc();
-    parser.reinitialize('response');
-    parser.socket = self;
-    parser.onIncoming = function (res) {
-      debug("incoming response!");
-
-      var req = self._outgoing[0];
-
-      // Responses to HEAD requests are AWFUL. Ask Ryan.
-      // A major oversight in HTTP. Hence this nastiness.
-      var isHeadResponse = req.method == "HEAD";
-      debug('isHeadResponse ' + isHeadResponse);
-
-      res.addListener('end', function ( ) {
-        debug("request complete disconnecting. readyState = " + self.readyState);
-        // For the moment we reconnect for every request. FIXME!
-        // All that should be required for keep-alive is to not reconnect,
-        // but outgoingFlush instead.
-        if (req.shouldKeepAlive) {
-          outgoingFlush(self)
-          self._outgoing.shift()
-          outgoingFlush(self)
-        } else {
-          self.end();
-        }
-      });
-
-      req.emit("response", res);
-
-      return isHeadResponse;
-    };
-  };
-
-  self.ondata = function (d, start, end) {
-    if (!parser) {
+  function onData(d, start, end) {
+    if (!self.parser) {
       throw new Error("parser not initialized prior to Client.ondata call");
     }
-    var ret = parser.execute(d, start, end - start);
+    var ret = self.parser.execute(d, start, end - start);
     if (ret instanceof Error) {
       self.destroy(ret);
-    } else if (parser.incoming && parser.incoming.upgrade) {
+    } else if (self.parser.incoming && self.parser.incoming.upgrade) {
       var bytesParsed = ret;
       self.ondata = null;
-      self.onend = null
+      self.onend = null;
 
-      var req = self._outgoing[0];
+      var req = self.parser.incoming;
 
       var upgradeHead = d.slice(start + bytesParsed + 1, end);
 
@@ -901,23 +908,27 @@ function Client ( ) {
 
   self.addListener("connect", function () {
     debug('client connected');
+
+    self.ondata = onData;
+    self.onend = onEnd;
+
     if (this.https) {
       this.setSecure(this.credentials);
     } else {
-      initParser();
-      debug('requests: ' + sys.inspect(self._outgoing));
+      self._initParser();
+      debug('requests: ' + util.inspect(self._outgoing));
       outgoingFlush(self);
     }
   });
 
   self.addListener("secure", function () {
-    initParser();
-    debug('requests: ' + sys.inspect(self._outgoing));
+    self._initParser();
+    debug('requests: ' + util.inspect(self._outgoing));
     outgoingFlush(self);
   });
 
-  self.onend = function () {
-    if (parser) parser.finish();
+  function onEnd() {
+    if (self.parser) self.parser.finish();
     debug("self got end closing. readyState = " + self.readyState);
     self.end();
   };
@@ -933,13 +944,13 @@ function Client ( ) {
     // If there are more requests to handle, reconnect.
     if (self._outgoing.length) {
       self._reconnect();
-    } else if (parser) {
-      parsers.free(parser);
-      parser = null;
+    } else if (self.parser) {
+      parsers.free(self.parser);
+      self.parser = null;
     }
   });
 };
-sys.inherits(Client, net.Stream);
+util.inherits(Client, net.Stream);
 
 exports.Client = Client;
 
@@ -953,6 +964,52 @@ exports.createClient = function (port, host, https, credentials) {
 };
 
 
+Client.prototype._initParser = function () {
+  var self = this;
+  if (!self.parser) self.parser = parsers.alloc();
+  self.parser.reinitialize('response');
+  self.parser.socket = self;
+  self.parser.onIncoming = function (res) {
+    debug("incoming response!");
+
+    var req = self._outgoing[0];
+
+    // Responses to HEAD requests are AWFUL. Ask Ryan.
+    // A major oversight in HTTP. Hence this nastiness.
+    var isHeadResponse = req.method == "HEAD";
+    debug('isHeadResponse ' + isHeadResponse);
+      
+    if (res.statusCode == 100) {
+      // restart the parser, as this is a continue message.
+      req.emit("continue");
+      return true;
+    }
+    
+    if (req.shouldKeepAlive && res.headers.connection === 'close') {
+      req.shouldKeepAlive = false;
+    }
+
+    res.addListener('end', function ( ) {
+      debug("request complete disconnecting. readyState = " + self.readyState);
+      // For the moment we reconnect for every request. FIXME!
+      // All that should be required for keep-alive is to not reconnect,
+      // but outgoingFlush instead.
+      if (req.shouldKeepAlive) {
+        outgoingFlush(self);
+        self._outgoing.shift();
+        outgoingFlush(self);
+      } else {
+        self.end();
+      }
+    });
+
+    req.emit("response", res);
+
+    return isHeadResponse;
+  };
+};
+
+
 // This is called each time a request has been pushed completely to the
 // socket. The message that was sent is still sitting at client._outgoing[0]
 // it is our responsibility to shift it off.
@@ -985,27 +1042,6 @@ Client.prototype._reconnect = function () {
   }
 };
 
-
-Client.prototype.get = function () {
-  throw new Error("client.get(...) is now client.request('GET', ...)");
-};
-
-Client.prototype.head = function () {
-  throw new Error("client.head(...) is now client.request('HEAD', ...)");
-};
-
-Client.prototype.post = function () {
-  throw new Error("client.post(...) is now client.request('POST', ...)");
-};
-
-Client.prototype.del = function () {
-  throw new Error("client.del(...) is now client.request('DELETE', ...)");
-};
-
-Client.prototype.put = function () {
-  throw new Error("client.put(...) is now client.request('PUT', ...)");
-};
-
 Client.prototype.request = function (method, url, headers) {
   if (typeof(url) != "string") {
     // assume method was omitted, shift arguments
