@@ -1,6 +1,7 @@
-var sys = require("sys");
+var util = require("util");
 var fs = require("fs");
 var events = require("events");
+var stream = require("stream");
 var dns = require('dns');
 
 var kMinPoolSpace = 128;
@@ -8,7 +9,7 @@ var kPoolSize = 40*1024;
 
 var debugLevel = parseInt(process.env.NODE_DEBUG, 16);
 function debug () {
-  if (debugLevel & 0x2) sys.error.apply(this, arguments);
+  if (debugLevel & 0x2) util.error.apply(this, arguments);
 }
 var binding = process.binding('net');
 
@@ -20,10 +21,10 @@ var binding = process.binding('net');
 // represent buffer.used) that can be seeked around would be easier. I'm not
 // yet convinced that every use-case can be fit into that abstraction, so
 // waiting to implement it until I get more experience with this.
-var Buffer = require('buffer').Buffer;
 var FreeList = require('freelist').FreeList;
 
-var IOWatcher   = process.IOWatcher;
+var IOWatcher   = process.binding('io_watcher').IOWatcher;
+var Timer       = process.binding('timer').Timer;
 var assert      = process.assert;
 
 var socket      = binding.socket;
@@ -47,15 +48,7 @@ var EINPROGRESS = binding.EINPROGRESS;
 var ENOENT      = binding.ENOENT;
 var EMFILE      = binding.EMFILE;
 var END_OF_FILE = 42;
-
-// Do we have openssl crypto?
-try {
-  var SecureContext = process.binding('crypto').SecureContext;
-  var SecureStream = process.binding('crypto').SecureStream;
-  var have_crypto = true;
-} catch (e) {
-  var have_crypto = false;
-}
+var SecureContext, SecureStream; // lazy loaded
 
 // IDLE TIMEOUTS
 //
@@ -99,7 +92,7 @@ var timeout = new (function () {
     remove(socket);
     socket._idleNext = list._idleNext;
     socket._idleNext._idlePrev = socket;
-    socket._idlePrev = list
+    socket._idlePrev = list;
     list._idleNext = socket;
   }
 
@@ -125,7 +118,7 @@ var timeout = new (function () {
     if (lists[msecs]) {
       list = lists[msecs];
     } else {
-      list = new process.Timer();
+      list = new Timer();
       list._idleNext = list;
       list._idlePrev = list;
 
@@ -208,7 +201,7 @@ var timeout = new (function () {
         socket._idlePrev._idleNext = socket._idleNext;
         socket._idleNext = list._idleNext;
         socket._idleNext._idlePrev = socket;
-        socket._idlePrev = list
+        socket._idlePrev = list;
         list._idleNext = socket;
       }
     }
@@ -219,6 +212,21 @@ var ioWatchers = new FreeList("iowatcher", 100, function () {
   return new IOWatcher();
 });
 
+exports.isIP = binding.isIP;
+
+exports.isIPv4 = function (input) {
+  if (binding.isIP(input) === 4) {
+    return true;
+  }
+  return false;
+};
+
+exports.isIPv6 = function (input) {
+  if (binding.isIP(input) === 6) {
+    return true;
+  }
+  return false;
+};
 
 // Allocated on demand.
 var pool = null;
@@ -241,7 +249,7 @@ function allocEmptyBuffer () {
 
 function _doFlush () {
   var socket = this.socket;
-  // Stream becomes writeable on connect() but don't flush if there's
+  // Stream becomes writable on connect() but don't flush if there's
   // nothing actually to write
   if (socket.flush()) {
     if (socket._events && socket._events['drain']) socket.emit("drain");
@@ -257,7 +265,7 @@ function setImplmentationMethods (self) {
   };
 
   if (self.type == 'unix') {
-    self._writeImpl = function(buf, off, len, fd, flags) {
+    self._writeImpl = function (buf, off, len, fd, flags) {
       // Detect and disallow zero-byte writes wth an attached file
       // descriptor. This is an implementation limitation of sendmsg(2).
       if (fd && noData(buf, off, len)) {
@@ -267,7 +275,7 @@ function setImplmentationMethods (self) {
       return sendMsg(self.fd, buf, off, len, fd, flags);
     };
 
-    self._readImpl = function(buf, off, len, calledByIOWatcher) {
+    self._readImpl = function (buf, off, len, calledByIOWatcher) {
       var bytesRead = recvMsg(self.fd, buf, off, len);
 
       // Do not emit this in the same stack, otherwise we risk corrupting our
@@ -279,18 +287,16 @@ function setImplmentationMethods (self) {
       // calling emit() after recvMsg() has been called again and end up
       // emitting null (or another FD).
       if (recvMsg.fd !== null) {
-        (function () {
-          var fd = recvMsg.fd;
-          process.nextTick(function() {
-            self.emit('fd', fd);
-          });
-        })();
+        var fd = recvMsg.fd;
+        process.nextTick(function () {
+          self.emit('fd', fd);
+        });
       }
 
       return bytesRead;
     };
   } else {
-    self._writeImpl = function(buf, off, len, fd, flags) {
+    self._writeImpl = function (buf, off, len, fd, flags) {
       // XXX: TLS support requires that 0-byte writes get processed
       //      by the kernel for some reason. Otherwise, we'd just
       //      fast-path return here.
@@ -300,30 +306,30 @@ function setImplmentationMethods (self) {
       return write(self.fd, buf, off, len);
     };
 
-    self._readImpl = function(buf, off, len, calledByIOWatcher) {
+    self._readImpl = function (buf, off, len, calledByIOWatcher) {
       return read(self.fd, buf, off, len);
     };
   }
 
-  self._shutdownImpl = function() {
-    shutdown(self.fd, 'write')
+  self._shutdownImpl = function () {
+    shutdown(self.fd, 'write');
   };
 
   if (self.secure) {
     var oldWrite = self._writeImpl;
-    self._writeImpl = function(buf, off, len, fd, flags) {
+    self._writeImpl = function (buf, off, len, fd, flags) {
       assert(buf);
       assert(self.secure);
 
-      var bytesWritten = self.secureStream.writeInject(buf, off, len);
+      var bytesWritten = self.secureStream.clearIn(buf, off, len);
 
       if (!securePool) {
         allocNewSecurePool();
       }
 
-      var secureLen = self.secureStream.writeExtract(
-        securePool, 0, securePool.length
-      );
+      var secureLen = self.secureStream.encOut(securePool,
+                                               0,
+                                               securePool.length);
 
       if (secureLen == -1) {
         // Check our read again for secure handshake
@@ -344,7 +350,7 @@ function setImplmentationMethods (self) {
     };
 
     var oldRead = self._readImpl;
-    self._readImpl = function(buf, off, len, calledByIOWatcher) {
+    self._readImpl = function (buf, off, len, calledByIOWatcher) {
       assert(self.secure);
 
       var bytesRead = 0;
@@ -356,17 +362,15 @@ function setImplmentationMethods (self) {
 
       if (calledByIOWatcher) {
         secureBytesRead = oldRead(securePool, 0, securePool.length);
-        self.secureStream.readInject(securePool, 0, secureBytesRead);
+        self.secureStream.encIn(securePool, 0, secureBytesRead);
       }
 
       var chunkBytes;
       do {
-        chunkBytes = self.secureStream.readExtract(
-          pool,
-          pool.used + bytesRead,
-          pool.length - pool.used - bytesRead
-        );
-
+        chunkBytes =
+          self.secureStream.clearOut(pool,
+                                     pool.used + bytesRead,
+                                     pool.length - pool.used - bytesRead);
         bytesRead += chunkBytes;
       } while ((chunkBytes > 0) && (pool.used + bytesRead < pool.length));
 
@@ -374,10 +378,9 @@ function setImplmentationMethods (self) {
         return -1;
       }
 
-      if (self.secureStream.readPending()) {
+      if (self.secureStream.clearPending()) {
         process.nextTick(function () {
-          if(self._readWatcher)
-            self._readWatcher.callback();
+          if (self._readWatcher) self._readWatcher.callback();
         });
       }
 
@@ -415,19 +418,17 @@ function setImplmentationMethods (self) {
     };
 
     var oldShutdown = self._shutdownImpl;
-    self._shutdownImpl = function() {
+    self._shutdownImpl = function () {
       self.secureStream.shutdown();
 
       if (!securePool) {
         allocNewSecurePool();
       }
 
-      var secureLen = self.secureStream.writeExtract(
-        securePool, 0, securePool.length
-      );
+      var len = self.secureStream.encOut(securePool, 0, securePool.length);
 
       try {
-        oldWrite(securePool, 0, secureLen);
+        oldWrite(securePool, 0, len);
       } catch (e) { }
 
       oldShutdown();
@@ -451,7 +452,10 @@ function initStream (self) {
     var bytesRead;
 
     try {
-      bytesRead = self._readImpl(pool, pool.used, pool.length - pool.used, (arguments.length > 0));
+      bytesRead = self._readImpl(pool,
+                                 pool.used,
+                                 pool.length - pool.used,
+                                 (arguments.length > 0));
     } catch (e) {
       self.destroy(e);
       return;
@@ -513,26 +517,33 @@ function initStream (self) {
 }
 
 function Stream (fd, type) {
-  events.EventEmitter.call(this);
+  if (!(this instanceof Stream)) return new Stream(fd, type);
+  stream.Stream.call(this);
 
   this.fd = null;
   this.type = null;
   this.secure = false;
 
-  if (parseInt(fd) >= 0) {
+  if (parseInt(fd, 10) >= 0) {
     this.open(fd, type);
   } else {
     setImplmentationMethods(this);
   }
 };
-sys.inherits(Stream, events.EventEmitter);
+util.inherits(Stream, stream.Stream);
 exports.Stream = Stream;
 
-Stream.prototype.setSecure = function(credentials) {
-  if (!have_crypto) {
+
+Stream.prototype.setSecure = function (credentials) {
+  // Do we have openssl crypto?
+  try {
+    SecureContext = process.binding('crypto').SecureContext;
+    SecureStream = process.binding('crypto').SecureStream;
+  } catch (e) {
     throw new Error('node.js not compiled with openssl crypto support.');
   }
-  var crypto= require("crypto");
+
+  var crypto = require("crypto");
   this.secure = true;
   this.secureEstablished = false;
   // If no credentials given, create a new one for just this Stream
@@ -542,10 +553,12 @@ Stream.prototype.setSecure = function(credentials) {
     this.credentials = credentials;
   }
   if (!this.server) {
-    // For clients, we will always have either a given ca list or the default one;
+    // For clients, we will always have either a given ca list or the default on
     this.credentials.shouldVerify = true;
   }
-  this.secureStream = new SecureStream(this.credentials.context, this.server ? 1 : 0, this.credentials.shouldVerify ? 1 : 0);
+  this.secureStream = new SecureStream(this.credentials.context,
+                                       this.server ? true : false,
+                                       this.credentials.shouldVerify);
 
   setImplmentationMethods(this);
 
@@ -553,18 +566,18 @@ Stream.prototype.setSecure = function(credentials) {
     // If client, trigger handshake
     this._checkForSecureHandshake();
   }
-}
+};
 
 
-Stream.prototype.verifyPeer = function() {
+Stream.prototype.verifyPeer = function () {
   if (!this.secure) {
     throw new Error('Stream is not a secure stream.');
   }
   return this.secureStream.verifyPeer(this.credentials.context);
-}
+};
 
 
-Stream.prototype._checkForSecureHandshake = function() {
+Stream.prototype._checkForSecureHandshake = function () {
   if (!this.writable) {
     return;
   }
@@ -572,23 +585,23 @@ Stream.prototype._checkForSecureHandshake = function() {
   // Do an empty write to see if we need to write out as part of handshake
   if (!emptyBuffer) allocEmptyBuffer();
   this.write(emptyBuffer);
-}
+};
 
 
-Stream.prototype.getPeerCertificate = function(credentials) {
+Stream.prototype.getPeerCertificate = function (credentials) {
   if (!this.secure) {
     throw new Error('Stream is not a secure stream.');
   }
   return this.secureStream.getPeerCertificate();
-}
+};
 
 
-Stream.prototype.getCipher = function() {
+Stream.prototype.getCipher = function () {
   if (!this.secure) {
       throw new Error('Stream is not a secure stream.');
   }
   return this.secureStream.getCurrentCipher();
-}
+};
 
 
 Stream.prototype.open = function (fd, type) {
@@ -602,7 +615,7 @@ Stream.prototype.open = function (fd, type) {
 
   this._writeWatcher.set(this.fd, false, true);
   this.writable = true;
-}
+};
 
 
 exports.createConnection = function (port, host) {
@@ -637,13 +650,21 @@ Object.defineProperty(Stream.prototype, 'readyState', {
 // something was queued. If data was queued, then the "drain" event will
 // signal when it has been finally flushed to socket.
 Stream.prototype.write = function (data, encoding, fd) {
-  if (this._writeQueue && this._writeQueue.length) {
+  if (this._connecting || (this._writeQueue && this._writeQueue.length)) {
+    if (!this._writeQueue) {
+      this._writeQueue = [];
+      this._writeQueueEncoding = [];
+      this._writeQueueFD = [];
+    }
+
     // Slow. There is already a write queue, so let's append to it.
     if (this._writeQueueLast() === END_OF_FILE) {
       throw new Error('Stream.end() called already; cannot write.');
     }
 
     if (typeof data == 'string' &&
+        this._writeQueue.length &&
+        typeof this._writeQueue[this._writeQueue.length-1] === 'string' &&
         this._writeQueueEncoding[this._writeQueueEncoding.length-1] === encoding) {
       // optimization - concat onto last
       this._writeQueue[this._writeQueue.length-1] += data;
@@ -689,7 +710,7 @@ Stream.prototype._writeOut = function (data, encoding, fd) {
     len = data.length;
 
   } else {
-    assert(typeof data == 'string')
+    assert(typeof data == 'string');
 
     if (!pool || pool.length - pool.used < kMinPoolSpace) {
       pool = null;
@@ -761,7 +782,7 @@ Stream.prototype._writeOut = function (data, encoding, fd) {
   var leftOver = buffer.slice(off + bytesWritten, off + len);
   leftOver.used = leftOver.length; // used the whole thing...
 
-  //  sys.error('data.used = ' + data.used);
+  //  util.error('data.used = ' + data.used);
   //if (!this._writeQueue) initWriteStream(this);
 
   // data should be the next thing to write.
@@ -774,7 +795,7 @@ Stream.prototype._writeOut = function (data, encoding, fd) {
   }
 
   return false;
-}
+};
 
 
 // Flushes the write buffer out.
@@ -798,11 +819,6 @@ Stream.prototype.flush = function () {
 };
 
 
-Stream.prototype.send = function () {
-  throw new Error('send renamed to write');
-};
-
-
 Stream.prototype._writeQueueLast = function () {
   return this._writeQueue.length > 0 ? this._writeQueue[this._writeQueue.length-1]
                                      : null;
@@ -849,13 +865,21 @@ function doConnect (socket, port, host) {
         socket.destroy(e);
         return;
       }
+
+
+      if (socket._writeQueue && socket._writeQueue.length) {
+        // Flush socket in case any writes are queued up while connecting.
+        // ugly
+        _doFlush.call(socket._writeWatcher);
+      }
+
     } else if (errno != EINPROGRESS) {
       socket.destroy(errnoException(errno, 'connect'));
     }
   };
 }
 
-function isPort (x) { return parseInt(x) >= 0; }
+function toPort (x) { return (x = Number(x)) >= 0 ? x : false; }
 
 
 // var stream = new Stream();
@@ -871,10 +895,18 @@ Stream.prototype.connect = function () {
   timeout.active(socket);
 
   self._connecting = true; // set false in doConnect
+  self.writable = true;
 
-  if (isPort(arguments[0])) {
+  var port = toPort(arguments[0]);
+  if (port === false) {
+    // UNIX
+    self.fd = socket('unix');
+    self.type = 'unix';
+
+    setImplmentationMethods(this);
+    doConnect(self, arguments[0]);
+  } else {
     // TCP
-    var port = arguments[0];
     dns.lookup(arguments[1], function (err, ip, addressType) {
       if (err) {
         self.emit('error', err);
@@ -884,13 +916,6 @@ Stream.prototype.connect = function () {
         doConnect(self, port, ip);
       }
     });
-  } else {
-    // UNIX
-    self.fd = socket('unix');
-    self.type = 'unix';
-
-    setImplmentationMethods(this);
-    doConnect(self, arguments[0]);
   }
 };
 
@@ -935,17 +960,6 @@ Stream.prototype.resume = function () {
 };
 
 
-var forceCloseWarning;
-
-Stream.prototype.forceClose = function (e) {
-  if (!forceCloseWarning) {
-    forceCloseWarning = "forceClose() has been renamed to destroy()";
-    sys.error(forceCloseWarning);
-  }
-  return this.destroy(e);
-};
-
-
 Stream.prototype.destroy = function (exception) {
   // pool is shared between sockets, so don't need to free it here.
   var self = this;
@@ -1010,28 +1024,22 @@ Stream.prototype._shutdown = function () {
   }
 };
 
-var closeDepricationWarning;
-
-Stream.prototype.close = function (data, encoding) {
-  if (!closeDepricationWarning) {
-    closeDepricationWarning = "Notification: Stream.prototype.close has been renamed to end()";
-    sys.error(closeDepricationWarning);
-  }
-  return this.end(data, encoding);
-};
 
 Stream.prototype.end = function (data, encoding) {
   if (this.writable) {
     if (data) this.write(data, encoding);
     if (this._writeQueueLast() !== END_OF_FILE) {
       this._writeQueue.push(END_OF_FILE);
-      this.flush();
+      if (!this._connecting) {
+        this.flush();
+      }
     }
   }
 };
 
 
 function Server (listener) {
+  if (!(this instanceof Server)) return new Server(listener);
   events.EventEmitter.call(this);
   var self = this;
 
@@ -1041,6 +1049,25 @@ function Server (listener) {
 
   self.connections = 0;
 
+  self.paused = false;
+  self.pauseTimeout = 1000;
+
+  function pause () {
+    // We've hit the maximum file limit. What to do?
+    // Let's try again in 1 second.
+    self.watcher.stop();
+
+    // If we're already paused, don't do another timeout.
+    if (self.paused) return;
+
+    setTimeout(function () {
+      self.paused = false;
+      // Make sure we haven't closed in the interim
+      if (typeof self.fd  != 'number') return;
+      self.watcher.start();
+    }, self.pauseTimeout);
+  }
+
   self.watcher = new IOWatcher();
   self.watcher.host = self;
   self.watcher.callback = function () {
@@ -1048,7 +1075,10 @@ function Server (listener) {
       try {
         var peerInfo = accept(self.fd);
       } catch (e) {
-        if (e.errno == EMFILE) return;
+        if (e.errno == EMFILE) {
+          pause();
+          return;
+        }
         throw e;
       }
       if (!peerInfo) return;
@@ -1081,7 +1111,7 @@ function Server (listener) {
     }
   };
 }
-sys.inherits(Server, events.EventEmitter);
+util.inherits(Server, events.EventEmitter);
 exports.Server = Server;
 
 
@@ -1107,7 +1137,8 @@ Server.prototype.listen = function () {
     self.addListener('listening', lastArg);
   }
 
-  if (!isPort(arguments[0])) {
+  var port = toPort(arguments[0]);
+  if (port === false) {
     // the first argument specifies a path
     self.fd = socket('unix');
     self.type = 'unix';
@@ -1117,8 +1148,7 @@ Server.prototype.listen = function () {
     fs.stat(path, function (err, r) {
       if (err) {
         if (err.errno == ENOENT) {
-          bind(self.fd, path);
-          self._doListen();
+          self._doListen(path);
         } else {
           throw r;
         }
@@ -1127,12 +1157,8 @@ Server.prototype.listen = function () {
           throw new Error("Non-socket exists at  " + path);
         } else {
           fs.unlink(path, function (err) {
-            if (err) {
-              throw err;
-            } else {
-              bind(self.fd, path);
-              self._doListen();
-            }
+            if (err) throw err;
+            self._doListen(path);
           });
         }
       }
@@ -1142,21 +1168,16 @@ Server.prototype.listen = function () {
     // The port can be found with server.address()
     self.type = 'tcp4';
     self.fd = socket(self.type);
-    bind(self.fd, arguments[0]);
-    process.nextTick(function () {
-      self._doListen();
-    });
+    self._doListen(port);
   } else {
     // the first argument is the port, the second an IP
-    var port = arguments[0];
     dns.lookup(arguments[1], function (err, ip, addressType) {
       if (err) {
         self.emit('error', err);
       } else {
         self.type = addressType == 4 ? 'tcp4' : 'tcp6';
         self.fd = socket(self.type);
-        bind(self.fd, port, ip);
-        self._doListen();
+        self._doListen(port, ip);
       }
     });
   }
@@ -1179,10 +1200,33 @@ Server.prototype._startWatcher = function () {
 };
 
 Server.prototype._doListen = function () {
-  listen(this.fd, 128);
-  this._startWatcher();
-}
+  var self = this;
 
+  try {
+    bind(self.fd, arguments[0], arguments[1]);
+  } catch (err) {
+    self.emit('error', err);
+    return;
+  }
+
+  // Need to the listening in the nextTick so that people potentially have
+  // time to register 'listening' listeners.
+  process.nextTick(function () {
+    // It could be that server.close() was called between the time the
+    // original listen command was issued and this. Bail if that's the case.
+    // See test/simple/test-net-eaddrinuse.js
+    if (typeof self.fd != 'number') return;
+
+    try {
+      listen(self.fd, self._backlog || 128);
+    } catch (err) {
+      self.emit('error', err);
+      return;
+    }
+
+    self._startWatcher();
+  });
+}
 
 
 Server.prototype.address = function () {
@@ -1207,5 +1251,3 @@ Server.prototype.close = function () {
     self.emit("close");
   }
 };
-
-// vim:ts=2 sw=2
