@@ -7,10 +7,8 @@
 var kHistorySize = 30;
 var kBufSize = 10*1024;
 
-
-var Buffer = require('buffer').Buffer;
-var sys = require('sys');
-var inherits = require('sys').inherits;
+var util = require('util');
+var inherits = require('util').inherits;
 var EventEmitter = require('events').EventEmitter;
 var stdio = process.binding('stdio');
 
@@ -30,7 +28,7 @@ function Interface (output, completer) {
 
   this.enabled = output.fd < 3;  // Looks like a TTY.
 
-  if (parseInt(process.env['NODE_NO_READLINE'])) {
+  if (parseInt(process.env['NODE_NO_READLINE'], 10)) {
     this.enabled = false;
   }
 
@@ -188,7 +186,7 @@ Interface.prototype._tabComplete = function () {
             var item = group[idx];
             self.output.write(item);
             if (col < maxColumns - 1) {
-              for (var s = 0; s < width - item.length; s++) {
+              for (var s = 0, itemLen = item.length; s < width - itemLen; s++) {
                 self.output.write(' ');
               }
             }
@@ -198,8 +196,8 @@ Interface.prototype._tabComplete = function () {
         self.output.write('\r\n');
       }
 
-      var group = [], c, i;
-      for (i = 0; i < completions.length; i++) {
+      var group = [], c;
+      for (var i = 0, compLen = completions.length; i < compLen; i++) {
         c = completions[i];
         if (c === "") {
           handleGroup(group);
@@ -213,7 +211,7 @@ Interface.prototype._tabComplete = function () {
       // If there is a common prefix to all matches, then apply that
       // portion.
       var prefix = commonPrefix(
-        completions.filter(function(e) { if (e) return e }));
+        completions.filter(function(e) { if (e) return e; }));
       if (prefix.length > completeOn.length) {
         self._insertString(prefix.slice(completeOn.length));
       }
@@ -230,7 +228,7 @@ function commonPrefix(strings) {
   var sorted = strings.slice().sort();
   var min = sorted[0];
   var max = sorted[sorted.length - 1];
-  for (var i = 0; i < min.length; i++) {
+  for (var i = 0, len = min.length; i < len; i++) {
     if (min[i] != max[i]) {
       return min.slice(0, i);
     }
@@ -475,7 +473,7 @@ Interface.prototype._ttyWrite = function (b) {
     default:
       var c = b.toString('utf8');
       var lines = c.split(/\r\n|\n|\r/);
-      for (var i = 0; i < lines.length; i++) {
+      for (var i = 0, len = lines.length; i < len; i++) {
         if (i > 0) {
           this._ttyWrite(new Buffer([13]));
         }
