@@ -12,48 +12,66 @@
 
 // repl.start("node > ").context.foo = "stdin is fun";  // expose foo to repl context
 
-var sys = require('sys');
+var util = require('util');
 var Script = process.binding('evals').Script;
 var evalcx = Script.runInContext;
 var path = require("path");
+var fs = require("fs");
 var rl = require('readline');
 var context;
 
-function cwdRequire (id) {
-  if (id.match(/^\.\.\//) || id.match(/^\.\//)) {
-    id = path.join(process.cwd(), id);
-  }
-  return require(id);
-}
-Object.keys(require).forEach(function (k) {
-  cwdRequire[k] = require[k];
-});
+var disableColors = process.env.NODE_DISABLE_COLORS ? true : false;
+
+// hack for require.resolve("./relative") to work properly.
+module.filename = process.cwd() + "/repl";
 
 function resetContext() {
   context = Script.createContext();
   for (var i in global) context[i] = global[i];
   context.module = module;
-  context.require = cwdRequire;
+  context.require = require;
 }
 
 
 // Can overridden with custom print functions, such as `probe` or `eyes.js`
-exports.writer = sys.inspect;
+exports.writer = util.inspect;
 
 function REPLServer(prompt, stream) {
   var self = this;
   if (!context) resetContext();
+  if (!exports.repl) exports.repl = this;
   self.context = context;
   self.buffered_cmd = '';
 
   self.stream = stream || process.openStdin();
-  self.prompt = prompt || "node> ";
+  self.prompt = prompt || "> ";
 
   var rli = self.rli = rl.createInterface(self.stream, function (text) {
     return self.complete(text);
   });
+
+  this.commands = {};
+  defineDefaultCommands(this);
+
+  if (rli.enabled && !disableColors) {
+    // Turn on ANSI coloring.
+    exports.writer = function(obj, showHidden, depth) {
+      return util.inspect(obj, showHidden, depth, true);
+    };
+  }
+
   rli.setPrompt(self.prompt);
 
+  rli.on("SIGINT", function () {
+    if (self.buffered_cmd && self.buffered_cmd.length > 0) {
+      rli.write("\n");
+      self.buffered_cmd = '';
+      self.displayPrompt();
+    } else {
+      rli.close();
+    }
+  });
+
   self.stream.addListener("data", function (chunk) {
     rli.write(chunk);
   });
@@ -65,7 +83,12 @@ function REPLServer(prompt, stream) {
 
     // Check to see if a REPL keyword was used. If it returns true,
     // display next prompt and return.
-    if (self.parseREPLKeyword(cmd) === true) return;
+    if (cmd && cmd.charAt(0) === '.') {
+      var matches = cmd.match(/^(\.[^\s]+)\s*(.*)$/);
+      var keyword = matches && matches[1];
+      var rest = matches && matches[2];
+      if (self.parseREPLKeyword(keyword, rest) === true) return;
+    }
 
     // The catchall for errors
     try {
@@ -129,7 +152,7 @@ exports.start = function (prompt, source) {
 };
 
 REPLServer.prototype.displayPrompt = function () {
-  this.rli.setPrompt(this.buffered_cmd.length ? '...   ' : this.prompt);
+  this.rli.setPrompt(this.buffered_cmd.length ? '... ' : this.prompt);
   this.rli.prompt();
 };
 
@@ -146,32 +169,97 @@ REPLServer.prototype.readline = function (cmd) {
  *    (2) the leading text completed.
  *
  * Example:
- *  complete('var foo = sys.')
- *    -> [['sys.print', 'sys.debug', 'sys.log', 'sys.inspect', 'sys.pump'],
- *        'sys.' ]
+ *  complete('var foo = util.')
+ *    -> [['util.print', 'util.debug', 'util.log', 'util.inspect', 'util.pump'],
+ *        'util.' ]
  *
- * TODO: add warning about exec'ing code... property getters could be run
+ * Warning: This eval's code like "foo.bar.baz", so it will run property
+ * getter code.
  */
 
 REPLServer.prototype.complete = function (line) {
-  // TODO: special completion in `require` calls.
-
   var completions,
       completionGroups = [],  // list of completion lists, one for each inheritance "level"
       completeOn,
       match, filter, i, j, group, c;
 
-  // REPL comments (e.g. ".break").
+  // REPL commands (e.g. ".break").
   var match = null;
   match = line.match(/^\s*(\.\w*)$/);
   if (match) {
-    completionGroups.push(['.break', '.clear', '.exit', '.help']);
+    completionGroups.push(Object.keys(this.commands));
     completeOn = match[1];
     if (match[1].length > 1) {
       filter = match[1];
     }
   }
 
+  // require('...<Tab>')
+  else if (match = line.match(/\brequire\s*\(['"](([\w\.\/-]+\/)?([\w\.\/-]*))/)) {
+    //TODO: suggest require.exts be exposed to be introspec registered extensions?
+    //TODO: suggest include the '.' in exts in internal repr: parity with `path.extname`.
+    var exts = [".js", ".node"];
+    var indexRe = new RegExp('^index(' + exts.map(regexpEscape).join('|') + ')$');
+
+    completeOn = match[1];
+    var subdir = match[2] || "";
+    var filter = match[1];
+    var dir, files, f, name, base, ext, abs, subfiles, s;
+    group = [];
+    for (i = 0; i < require.paths.length; i++) {
+      dir = require.paths[i];
+      if (subdir && subdir[0] === '/') {
+        dir = subdir;
+      } else if (subdir) {
+        dir = path.join(dir, subdir);
+      }
+      try {
+        files = fs.readdirSync(dir);
+      } catch (e) {
+        continue;
+      }
+      for (f = 0; f < files.length; f++) {
+        name = files[f];
+        ext = path.extname(name);
+        base = name.slice(0, -ext.length);
+        if (base.match(/-\d+\.\d+(\.\d+)?/) || name === ".npm") {
+          // Exclude versioned names that 'npm' installs.
+          continue;
+        }
+        if (exts.indexOf(ext) !== -1) {
+          if (!subdir || base !== "index") {
+            group.push(subdir + base);
+          }
+        } else {
+          abs = path.join(dir, name);
+          try {
+            if (fs.statSync(abs).isDirectory()) {
+              group.push(subdir + name + '/');
+              subfiles = fs.readdirSync(abs);
+              for (s = 0; s < subfiles.length; s++) {
+                if (indexRe.test(subfiles[s])) {
+                  group.push(subdir + name);
+                }
+              }
+            }
+          } catch(e) {}
+        }
+      }
+    }
+    if (group.length) {
+      completionGroups.push(group);
+    }
+
+    if (!subdir) {
+      // Kind of lame that this needs to be updated manually.
+      // Intentionally excluding moved modules: posix, utils.
+      var builtinLibs = ['assert', 'buffer', 'child_process', 'crypto', 'dgram',
+        'dns', 'events', 'file', 'freelist', 'fs', 'http', 'net', 'path',
+        'querystring', 'readline', 'repl', 'string_decoder', 'util', 'tcp', 'url'];
+      completionGroups.push(builtinLibs);
+    }
+  }
+
   // Handle variable member lookup.
   // We support simple chained expressions like the following (no function
   // calls, etc.). That is for simplicity and also because we *eval* that
@@ -182,8 +270,8 @@ REPLServer.prototype.complete = function (line) {
   //   spam.eggs.<|>  # completions for 'spam.eggs' with filter ''
   //   foo<|>         # all scope vars with filter 'foo'
   //   foo.<|>        # completions for 'foo' with filter ''
-  else if (line.length === 0 || line[line.length-1].match(/\w|\./)) {
-    var simpleExpressionPat = /(([a-zA-Z_]\w*)\.)*([a-zA-Z_]\w*)\.?$/;
+  else if (line.length === 0 || line[line.length-1].match(/\w|\.|\$/)) {
+    var simpleExpressionPat = /(([a-zA-Z_$](?:\w|\$)*)\.)*([a-zA-Z_$](?:\w|\$)*)\.?$/;
     match = simpleExpressionPat.exec(line);
     if (line.length === 0 || match) {
       var expr;
@@ -222,7 +310,7 @@ REPLServer.prototype.complete = function (line) {
             "false", "finally", "for", "function", "if", "import", "in",
             "instanceof", "let", "new", "null", "return", "switch", "this",
             "throw", "true", "try", "typeof", "undefined", "var", "void",
-            "while", "with", "yield"])
+            "while", "with", "yield"]);
         }
       } else {
         try {
@@ -231,12 +319,11 @@ REPLServer.prototype.complete = function (line) {
           //console.log("completion eval error, expr='"+expr+"': "+e);
         }
         if (obj != null) {
-          //TODO: The following, for example, misses "Object.isSealed". Is there
-          //      a way to introspec those? Need to hardcode?
           if (typeof obj === "object" || typeof obj === "function") {
             memberGroups.push(Object.getOwnPropertyNames(obj));
           }
-          var p = obj.constructor.prototype; // works for non-objects
+          // works for non-objects
+          var p = obj.constructor ? obj.constructor.prototype : null;
           try {
             var sentinel = 5;
             while (p !== null) {
@@ -312,34 +399,63 @@ REPLServer.prototype.complete = function (line) {
  * @returns {Boolean} If true it means don't continue parsing the command
  */
 
-REPLServer.prototype.parseREPLKeyword = function (cmd) {
-  var self = this;
-
-  switch (cmd) {
-  case ".break":
-    self.buffered_cmd = '';
-    self.displayPrompt();
-    return true;
-  case ".clear":
-    self.stream.write("Clearing context...\n");
-    self.buffered_cmd = '';
-    resetContext();
-    self.displayPrompt();
-    return true;
-  case ".exit":
-    self.stream.destroy();
-    return true;
-  case ".help":
-    self.stream.write(".break\tSometimes you get stuck in a place you can't get out... This will get you out.\n");
-    self.stream.write(".clear\tBreak, and also clear the local context.\n");
-    self.stream.write(".exit\tExit the prompt\n");
-    self.stream.write(".help\tShow repl options\n");
-    self.displayPrompt();
+REPLServer.prototype.parseREPLKeyword = function (keyword, rest) {
+  var cmd = this.commands[keyword];
+  if (cmd) {
+    cmd.action.call(this, rest);
     return true;
   }
   return false;
 };
 
+REPLServer.prototype.defineCommand = function(keyword, cmd) {
+  if (typeof cmd === 'function') cmd = {action: cmd};
+  else if (typeof cmd.action !== 'function') {
+    throw new Error('bad argument, action must be a function');
+  }
+  this.commands['.' + keyword] = cmd;
+};
+
+function defineDefaultCommands(repl) {
+  // TODO remove me after 0.3.x
+  repl.defineCommand('break', {
+    help: 'Sometimes you get stuck, this gets you out',
+    action: function() {
+      this.buffered_cmd = '';
+      this.displayPrompt();
+    }
+  });
+
+  repl.defineCommand('clear', {
+    help: 'Break, and also clear the local context',
+    action: function() {
+      this.stream.write("Clearing context...\n");
+      this.buffered_cmd = '';
+      resetContext();
+      this.displayPrompt();
+    }
+  });
+
+  repl.defineCommand('exit', {
+    help: 'Exit the repl',
+    action: function() {
+      this.stream.destroy();
+    }
+  });
+
+  repl.defineCommand('help', {
+    help: 'Show repl options',
+    action: function() {
+      var self = this;
+      Object.keys(this.commands).sort().forEach(function(name) {
+        var cmd = self.commands[name];
+        self.stream.write(name + "\t" + (cmd.help || '') + "\n");
+      });
+      this.displayPrompt();
+    }
+  });
+}
+
 function trimWhitespace (cmd) {
   var trimmer = /^\s*(.+)\s*$/m,
     matches = trimmer.exec(cmd);
@@ -349,6 +465,10 @@ function trimWhitespace (cmd) {
   }
 }
 
+function regexpEscape(s) {
+  return s.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&");
+}
+
 /**
  * Converts commands that use var and function <name>() to use the
  * local exports.context when evaled. This provides a local context
