@@ -22,31 +22,6 @@ namespace node {
 using namespace v8;
 
 
-static Handle<Value> IsIP(const Arguments& args) {
-  HandleScope scope;
-  
-  if (!args[0]->IsString()) {
-    return scope.Close(Integer::New(4));
-  }
-
-  String::Utf8Value s(args[0]->ToString());
-
-  // avoiding buffer overflows in the following strcat
-  // 2001:0db8:85a3:08d3:1319:8a2e:0370:7334
-  // 39 = max ipv6 address.
-  if (s.length() > INET6_ADDRSTRLEN) {
-    return scope.Close(Integer::New(0));
-  }
-
-  struct sockaddr_in6 a;
-
-  if (inet_pton(AF_INET, *s, &(a.sin6_addr)) > 0) return scope.Close(Integer::New(4));
-  if (inet_pton(AF_INET6, *s, &(a.sin6_addr)) > 0) return scope.Close(Integer::New(6));
-
-  return scope.Close(Integer::New(0));
-}
-
-
 class Channel : public ObjectWrap {
  public:
   static void Initialize(Handle<Object> target);
@@ -139,8 +114,6 @@ void Cares::Initialize(Handle<Object> target) {
   target->Set(String::NewSymbol("EREFUSED"), Integer::New(ARES_EREFUSED));
   target->Set(String::NewSymbol("SERVFAIL"), Integer::New(ARES_ESERVFAIL));
 
-  NODE_SET_METHOD(target, "isIP", IsIP);
-
   Channel::Initialize(target);
 }
 
@@ -223,7 +196,7 @@ static void ResolveError(Persistent<Function> &cb, int status) {
 
   TryCatch try_catch;
 
-  cb->Call(Context::GetCurrent()->Global(), 1, &e);
+  cb->Call(v8::Context::GetCurrent()->Global(), 1, &e);
 
   if (try_catch.HasCaught()) {
     FatalException(try_catch);
@@ -251,7 +224,7 @@ static void HostByNameCb(void *data,
 
   Local<Value> argv[2] = { Local<Value>::New(Null()), addresses};
 
-  (*cb)->Call(Context::GetCurrent()->Global(), 2, argv);
+  (*cb)->Call(v8::Context::GetCurrent()->Global(), 2, argv);
 
   if (try_catch.HasCaught()) {
     FatalException(try_catch);
@@ -281,7 +254,7 @@ static void HostByAddrCb(void *data,
 
   Local<Value> argv[2] = { Local<Value>::New(Null()), names };
 
-  (*cb)->Call(Context::GetCurrent()->Global(), 2, argv);
+  (*cb)->Call(v8::Context::GetCurrent()->Global(), 2, argv);
 
   if (try_catch.HasCaught()) {
     FatalException(try_catch);
@@ -294,7 +267,7 @@ static void HostByAddrCb(void *data,
 static void cb_call(Persistent<Function> &cb, int argc, Local<Value> *argv) {
   TryCatch try_catch;
 
-  cb->Call(Context::GetCurrent()->Global(), argc, argv);
+  cb->Call(v8::Context::GetCurrent()->Global(), argc, argv);
 
   if (try_catch.HasCaught()) {
     FatalException(try_catch);
@@ -486,6 +459,10 @@ void Channel::Initialize(Handle<Object> target) {
 
 
 Handle<Value> Channel::New(const Arguments& args) {
+  if (!args.IsConstructCall()) {
+    return FromConstructorTemplate(constructor_template, args);
+  }
+
   HandleScope scope;
 
   struct ares_options options;
@@ -761,3 +738,5 @@ void Channel::SockStateCb(void *data, int sock, int read, int write) {
 
 
 }  // namespace node
+
+NODE_MODULE(node_cares, node::Cares::Initialize);
