@@ -31,6 +31,7 @@ static Persistent<String> version_symbol;
 
 
 static int verify_callback(int ok, X509_STORE_CTX *ctx) {
+  assert(ok);
   return(1); // Ignore errors by now. VerifyPeer will catch them by using SSL_get_verify_result.
 }
 
@@ -100,13 +101,13 @@ Handle<Value> SecureContext::Init(const Arguments& args) {
       method = TLSv1_client_method();
   }
 
-  sc->pCtx = SSL_CTX_new(method);
+  sc->ctx_ = SSL_CTX_new(method);
   // Enable session caching?
-  SSL_CTX_set_session_cache_mode(sc->pCtx, SSL_SESS_CACHE_SERVER);
-  // SSL_CTX_set_session_cache_mode(sc->pCtx,SSL_SESS_CACHE_OFF);
+  SSL_CTX_set_session_cache_mode(sc->ctx_, SSL_SESS_CACHE_SERVER);
+  // SSL_CTX_set_session_cache_mode(sc->ctx_,SSL_SESS_CACHE_OFF);
 
-  sc->caStore = X509_STORE_new();
-  SSL_CTX_set_cert_store(sc->pCtx, sc->caStore);
+  sc->ca_store_ = X509_STORE_new();
+  SSL_CTX_set_cert_store(sc->ctx_, sc->ca_store_);
   return True();
 }
 
@@ -116,25 +117,29 @@ Handle<Value> SecureContext::SetKey(const Arguments& args) {
 
   SecureContext *sc = ObjectWrap::Unwrap<SecureContext>(args.Holder());
 
-  if (args.Length() != 1 ||
-      !args[0]->IsString()) {
-    return ThrowException(Exception::TypeError(
-          String::New("Bad parameter")));
+  if (args.Length() != 1 || !args[0]->IsString()) {
+    return ThrowException(Exception::TypeError(String::New("Bad parameter")));
   }
-  String::Utf8Value keyPem(args[0]->ToString());
 
-  BIO *bp = NULL;
-  EVP_PKEY* pkey;
-  bp = BIO_new(BIO_s_mem());
-  if (!BIO_write(bp, *keyPem, strlen(*keyPem)))
+  String::Utf8Value key_pem(args[0]->ToString());
+
+  BIO *bp = BIO_new(BIO_s_mem());
+
+  if (!BIO_write(bp, *key_pem, key_pem.length())) {
+    BIO_free(bp);
     return False();
+  }
 
-  pkey = PEM_read_bio_PrivateKey(bp, NULL, NULL, NULL);
-  if (pkey == NULL)
+  EVP_PKEY* pkey = PEM_read_bio_PrivateKey(bp, NULL, NULL, NULL);
+
+  if (pkey == NULL) {
+    BIO_free(bp);
     return False();
+  }
 
-  SSL_CTX_use_PrivateKey(sc->pCtx, pkey);
+  SSL_CTX_use_PrivateKey(sc->ctx_, pkey);
   BIO_free(bp);
+  // XXX Free pkey? 
 
   return True();
 }
@@ -150,19 +155,24 @@ Handle<Value> SecureContext::SetCert(const Arguments& args) {
     return ThrowException(Exception::TypeError(
           String::New("Bad parameter")));
   }
-  String::Utf8Value certPem(args[0]->ToString());
+  String::Utf8Value cert_pem(args[0]->ToString());
 
-  BIO *bp = NULL;
-  X509 *        x509;
-  bp = BIO_new(BIO_s_mem());
-  if (!BIO_write(bp, *certPem, strlen(*certPem)))
+  BIO *bp = BIO_new(BIO_s_mem());
+
+  if (!BIO_write(bp, *cert_pem, cert_pem.length())) {
+    BIO_free(bp);
     return False();
+  }
 
-  x509 = PEM_read_bio_X509(bp, NULL, NULL, NULL);
-  if (x509 == NULL)
+  X509 * x509 = PEM_read_bio_X509(bp, NULL, NULL, NULL);
+
+  if (x509 == NULL) {
+    BIO_free(bp);
     return False();
+  }
+
+  SSL_CTX_use_certificate(sc->ctx_, x509);
 
-  SSL_CTX_use_certificate(sc->pCtx, x509);
   BIO_free(bp);
   X509_free(x509);
 
@@ -175,24 +185,26 @@ Handle<Value> SecureContext::AddCACert(const Arguments& args) {
 
   SecureContext *sc = ObjectWrap::Unwrap<SecureContext>(args.Holder());
 
-  if (args.Length() != 1 ||
-      !args[0]->IsString()) {
-    return ThrowException(Exception::TypeError(
-          String::New("Bad parameter")));
+  if (args.Length() != 1 || !args[0]->IsString()) {
+    return ThrowException(Exception::TypeError(String::New("Bad parameter")));
   }
-  String::Utf8Value certPem(args[0]->ToString());
+  String::Utf8Value cert_pem(args[0]->ToString());
 
-  BIO *bp = NULL;
-  X509 *x509;
-  bp = BIO_new(BIO_s_mem());
-  if (!BIO_write(bp, *certPem, strlen(*certPem)))
+  BIO *bp = BIO_new(BIO_s_mem());
+
+  if (!BIO_write(bp, *cert_pem, cert_pem.length())) {
+    BIO_free(bp);
     return False();
+  }
 
-  x509 = PEM_read_bio_X509(bp, NULL, NULL, NULL);
-  if (x509 == NULL)
+  X509 *x509 = PEM_read_bio_X509(bp, NULL, NULL, NULL);
+
+  if (x509 == NULL) {
+    BIO_free(bp);
     return False();
+  }
 
-  X509_STORE_add_cert(sc->caStore, x509);
+  X509_STORE_add_cert(sc->ca_store_, x509);
 
   BIO_free(bp);
   X509_free(x509);
@@ -206,13 +218,12 @@ Handle<Value> SecureContext::SetCiphers(const Arguments& args) {
 
   SecureContext *sc = ObjectWrap::Unwrap<SecureContext>(args.Holder());
 
-  if (args.Length() != 1 ||
-      !args[0]->IsString()) {
-    return ThrowException(Exception::TypeError(
-          String::New("Bad parameter")));
+  if (args.Length() != 1 || !args[0]->IsString()) {
+    return ThrowException(Exception::TypeError(String::New("Bad parameter")));
   }
+
   String::Utf8Value ciphers(args[0]->ToString());
-  SSL_CTX_set_cipher_list(sc->pCtx, *ciphers);
+  SSL_CTX_set_cipher_list(sc->ctx_, *ciphers);
 
   return True();
 }
@@ -223,10 +234,11 @@ Handle<Value> SecureContext::Close(const Arguments& args) {
 
   SecureContext *sc = ObjectWrap::Unwrap<SecureContext>(args.Holder());
 
-  if (sc->pCtx != NULL) {
-    SSL_CTX_free(sc->pCtx);
+  if (sc->ctx_ != NULL) {
+    SSL_CTX_free(sc->ctx_);
     return True();
   }
+
   return False();
 }
 
@@ -238,30 +250,18 @@ void SecureStream::Initialize(Handle<Object> target) {
   t->InstanceTemplate()->SetInternalFieldCount(1);
   t->SetClassName(String::NewSymbol("SecureStream"));
 
-  NODE_SET_PROTOTYPE_METHOD(t, "readInject",
-                            SecureStream::ReadInject);
-  NODE_SET_PROTOTYPE_METHOD(t, "readExtract",
-                            SecureStream::ReadExtract);
-  NODE_SET_PROTOTYPE_METHOD(t, "writeInject",
-                            SecureStream::WriteInject);
-  NODE_SET_PROTOTYPE_METHOD(t, "writeExtract",
-                            SecureStream::WriteExtract);
-  NODE_SET_PROTOTYPE_METHOD(t, "readPending",
-                            SecureStream::ReadPending);
-  NODE_SET_PROTOTYPE_METHOD(t, "writeCanExtract",
-                            SecureStream::WriteCanExtract);
-  NODE_SET_PROTOTYPE_METHOD(t, "getPeerCertificate",
-                            SecureStream::GetPeerCertificate);
-  NODE_SET_PROTOTYPE_METHOD(t, "isInitFinished",
-                            SecureStream::IsInitFinished);
-  NODE_SET_PROTOTYPE_METHOD(t, "verifyPeer",
-                            SecureStream::VerifyPeer);
-  NODE_SET_PROTOTYPE_METHOD(t, "getCurrentCipher",
-                            SecureStream::GetCurrentCipher);
-  NODE_SET_PROTOTYPE_METHOD(t, "shutdown",
-                            SecureStream::Shutdown);
-  NODE_SET_PROTOTYPE_METHOD(t, "close",
-                            SecureStream::Close);
+  NODE_SET_PROTOTYPE_METHOD(t, "encIn", SecureStream::EncIn);
+  NODE_SET_PROTOTYPE_METHOD(t, "clearOut", SecureStream::ClearOut);
+  NODE_SET_PROTOTYPE_METHOD(t, "clearIn", SecureStream::ClearIn);
+  NODE_SET_PROTOTYPE_METHOD(t, "encOut", SecureStream::EncOut);
+  NODE_SET_PROTOTYPE_METHOD(t, "clearPending", SecureStream::ClearPending);
+  NODE_SET_PROTOTYPE_METHOD(t, "encPending", SecureStream::EncPending);
+  NODE_SET_PROTOTYPE_METHOD(t, "getPeerCertificate", SecureStream::GetPeerCertificate);
+  NODE_SET_PROTOTYPE_METHOD(t, "isInitFinished", SecureStream::IsInitFinished);
+  NODE_SET_PROTOTYPE_METHOD(t, "verifyPeer", SecureStream::VerifyPeer);
+  NODE_SET_PROTOTYPE_METHOD(t, "getCurrentCipher", SecureStream::GetCurrentCipher);
+  NODE_SET_PROTOTYPE_METHOD(t, "shutdown", SecureStream::Shutdown);
+  NODE_SET_PROTOTYPE_METHOD(t, "close", SecureStream::Close);
 
   target->Set(String::NewSymbol("SecureStream"), t->GetFunction());
 }
@@ -269,46 +269,40 @@ void SecureStream::Initialize(Handle<Object> target) {
 
 Handle<Value> SecureStream::New(const Arguments& args) {
   HandleScope scope;
+
   SecureStream *p = new SecureStream();
   p->Wrap(args.Holder());
 
-  if (args.Length() <1 ||
-      !args[0]->IsObject()) {
+  if (args.Length() < 1 || !args[0]->IsObject()) {
     return ThrowException(Exception::Error(String::New(
       "First argument must be a crypto module Credentials")));
   }
+
   SecureContext *sc = ObjectWrap::Unwrap<SecureContext>(args[0]->ToObject());
-  int isServer = 0;
-  int shouldVerify = 0;
-  if (args.Length() >=2 &&
-      args[1]->IsNumber()) {
-    isServer = args[1]->Int32Value();
-  }
-  if (args.Length() >=3 &&
-      args[2]->IsNumber()) {
-    shouldVerify = args[2]->Int32Value();
+
+  bool is_server = args[1]->BooleanValue();
+  bool should_verify = args[2]->BooleanValue();
+
+  p->ssl_ = SSL_new(sc->ctx_);
+  p->bio_read_ = BIO_new(BIO_s_mem());
+  p->bio_write_ = BIO_new(BIO_s_mem());
+  SSL_set_bio(p->ssl_, p->bio_read_, p->bio_write_);
+
+  if ((p->should_verify_ = should_verify)) {
+    SSL_set_verify(p->ssl_, SSL_VERIFY_PEER, verify_callback);
   }
 
-  p->pSSL = SSL_new(sc->pCtx);
-  p->pbioRead = BIO_new(BIO_s_mem());
-  p->pbioWrite = BIO_new(BIO_s_mem());
-  SSL_set_bio(p->pSSL, p->pbioRead, p->pbioWrite);
-  p->shouldVerify = shouldVerify>0;
-  if (p->shouldVerify) {
-    SSL_set_verify(p->pSSL, SSL_VERIFY_PEER, verify_callback);
-  }
-  p->server = isServer>0;
-  if (p->server) {
-    SSL_set_accept_state(p->pSSL);
+  if ((p->is_server_ = is_server)) {
+    SSL_set_accept_state(p->ssl_);
   } else {
-    SSL_set_connect_state(p->pSSL);
+    SSL_set_connect_state(p->ssl_);
   }
 
   return args.This();
 }
 
 
-Handle<Value> SecureStream::ReadInject(const Arguments& args) {
+Handle<Value> SecureStream::EncIn(const Arguments& args) {
   HandleScope scope;
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
@@ -323,21 +317,23 @@ Handle<Value> SecureStream::ReadInject(const Arguments& args) {
           String::New("Second argument should be a buffer")));
   }
 
-  Buffer * buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
+  Local<Object> buffer_obj = args[0]->ToObject();
+  char *buffer_data = Buffer::Data(buffer_obj);
+  size_t buffer_length = Buffer::Length(buffer_obj);
 
   size_t off = args[1]->Int32Value();
-  if (off >= buffer->length()) {
+  if (off >= buffer_length) {
     return ThrowException(Exception::Error(
           String::New("Offset is out of bounds")));
   }
 
   size_t len = args[2]->Int32Value();
-  if (off + len > buffer->length()) {
+  if (off + len > buffer_length) {
     return ThrowException(Exception::Error(
           String::New("Length is extends beyond buffer")));
   }
 
-  int bytes_written = BIO_write(ss->pbioRead, (char*)buffer->data() + off, len);
+  int bytes_written = BIO_write(ss->bio_read_, (char*)buffer_data + off, len);
 
   if (bytes_written < 0) {
     if (errno == EAGAIN || errno == EINTR) return Null();
@@ -348,7 +344,7 @@ Handle<Value> SecureStream::ReadInject(const Arguments& args) {
 }
 
 
-Handle<Value> SecureStream::ReadExtract(const Arguments& args) {
+Handle<Value> SecureStream::ClearOut(const Arguments& args) {
   HandleScope scope;
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
@@ -363,40 +359,42 @@ Handle<Value> SecureStream::ReadExtract(const Arguments& args) {
           String::New("Second argument should be a buffer")));
   }
 
-  Buffer * buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
+  Local<Object> buffer_obj = args[0]->ToObject();
+  char *buffer_data = Buffer::Data(buffer_obj);
+  size_t buffer_length = Buffer::Length(buffer_obj);
 
   size_t off = args[1]->Int32Value();
-  if (off >= buffer->length()) {
+  if (off >= buffer_length) {
     return ThrowException(Exception::Error(
           String::New("Offset is out of bounds")));
   }
 
   size_t len = args[2]->Int32Value();
-  if (off + len > buffer->length()) {
+  if (off + len > buffer_length) {
     return ThrowException(Exception::Error(
           String::New("Length is extends beyond buffer")));
   }
 
   int bytes_read;
 
-  if (!SSL_is_init_finished(ss->pSSL)) {
-    if (ss->server) {
-      bytes_read = SSL_accept(ss->pSSL);
+  if (!SSL_is_init_finished(ss->ssl_)) {
+    if (ss->is_server_) {
+      bytes_read = SSL_accept(ss->ssl_);
     } else {
-      bytes_read = SSL_connect(ss->pSSL);
+      bytes_read = SSL_connect(ss->ssl_);
     }
     if (bytes_read < 0) {
       int err;
-      if ((err = SSL_get_error(ss->pSSL, bytes_read)) == SSL_ERROR_WANT_READ) {
+      if ((err = SSL_get_error(ss->ssl_, bytes_read)) == SSL_ERROR_WANT_READ) {
         return scope.Close(Integer::New(0));
       }
     }
     return scope.Close(Integer::New(0));
   }
 
-  bytes_read = SSL_read(ss->pSSL, (char*)buffer->data() + off, len);
+  bytes_read = SSL_read(ss->ssl_, (char*)buffer_data + off, len);
   if (bytes_read < 0) {
-    int err = SSL_get_error(ss->pSSL, bytes_read);
+    int err = SSL_get_error(ss->ssl_, bytes_read);
     if (err == SSL_ERROR_WANT_READ) {
       return scope.Close(Integer::New(0));
     }
@@ -413,25 +411,25 @@ Handle<Value> SecureStream::ReadExtract(const Arguments& args) {
 }
 
 
-Handle<Value> SecureStream::ReadPending(const Arguments& args) {
+Handle<Value> SecureStream::ClearPending(const Arguments& args) {
   HandleScope scope;
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
-  int bytes_pending = BIO_pending(ss->pbioRead);
+  int bytes_pending = BIO_pending(ss->bio_read_);
   return scope.Close(Integer::New(bytes_pending));
 }
 
 
-Handle<Value> SecureStream::WriteCanExtract(const Arguments& args) {
+Handle<Value> SecureStream::EncPending(const Arguments& args) {
   HandleScope scope;
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
-  int bytes_pending = BIO_pending(ss->pbioWrite);
+  int bytes_pending = BIO_pending(ss->bio_write_);
   return scope.Close(Integer::New(bytes_pending));
 }
 
 
-Handle<Value> SecureStream::WriteExtract(const Arguments& args) {
+Handle<Value> SecureStream::EncOut(const Arguments& args) {
   HandleScope scope;
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
@@ -446,27 +444,29 @@ Handle<Value> SecureStream::WriteExtract(const Arguments& args) {
           String::New("Second argument should be a buffer")));
   }
 
-  Buffer * buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
+  Local<Object> buffer_obj = args[0]->ToObject();
+  char *buffer_data = Buffer::Data(buffer_obj);
+  size_t buffer_length = Buffer::Length(buffer_obj);
 
   size_t off = args[1]->Int32Value();
-  if (off >= buffer->length()) {
+  if (off >= buffer_length) {
     return ThrowException(Exception::Error(
           String::New("Offset is out of bounds")));
   }
 
   size_t len = args[2]->Int32Value();
-  if (off + len > buffer->length()) {
+  if (off + len > buffer_length) {
     return ThrowException(Exception::Error(
           String::New("Length is extends beyond buffer")));
   }
 
-  int bytes_read = BIO_read(ss->pbioWrite, (char*)buffer->data() + off, len);
+  int bytes_read = BIO_read(ss->bio_write_, (char*)buffer_data + off, len);
 
   return scope.Close(Integer::New(bytes_read));
 }
 
 
-Handle<Value> SecureStream::WriteInject(const Arguments& args) {
+Handle<Value> SecureStream::ClearIn(const Arguments& args) {
   HandleScope scope;
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
@@ -481,30 +481,32 @@ Handle<Value> SecureStream::WriteInject(const Arguments& args) {
           String::New("Second argument should be a buffer")));
   }
 
-  Buffer * buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
+  Local<Object> buffer_obj = args[0]->ToObject();
+  char *buffer_data = Buffer::Data(buffer_obj);
+  size_t buffer_length = Buffer::Length(buffer_obj);
 
   size_t off = args[1]->Int32Value();
-  if (off >= buffer->length()) {
+  if (off >= buffer_length) {
     return ThrowException(Exception::Error(
           String::New("Offset is out of bounds")));
   }
 
   size_t len = args[2]->Int32Value();
-  if (off + len > buffer->length()) {
+  if (off + len > buffer_length) {
     return ThrowException(Exception::Error(
           String::New("Length is extends beyond buffer")));
   }
 
-  if (!SSL_is_init_finished(ss->pSSL)) {
+  if (!SSL_is_init_finished(ss->ssl_)) {
     int s;
-    if (ss->server) {
-      s = SSL_accept(ss->pSSL);
+    if (ss->is_server_) {
+      s = SSL_accept(ss->ssl_);
     } else {
-      s = SSL_connect(ss->pSSL);
+      s = SSL_connect(ss->ssl_);
     }
     return scope.Close(Integer::New(0));
   }
-  int bytes_written = SSL_write(ss->pSSL, (char*)buffer->data() + off, len);
+  int bytes_written = SSL_write(ss->ssl_, (char*)buffer_data + off, len);
 
   return scope.Close(Integer::New(bytes_written));
 }
@@ -515,9 +517,9 @@ Handle<Value> SecureStream::GetPeerCertificate(const Arguments& args) {
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
 
-  if (ss->pSSL == NULL) return Undefined();
+  if (ss->ssl_ == NULL) return Undefined();
   Local<Object> info = Object::New();
-  X509* peer_cert = SSL_get_peer_certificate(ss->pSSL);
+  X509* peer_cert = SSL_get_peer_certificate(ss->ssl_);
   if (peer_cert != NULL) {
     char* subject = X509_NAME_oneline(X509_get_subject_name(peer_cert), 0, 0);
     if (subject != NULL) {
@@ -574,8 +576,8 @@ Handle<Value> SecureStream::Shutdown(const Arguments& args) {
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
 
-  if (ss->pSSL == NULL) return False();
-  if (SSL_shutdown(ss->pSSL) == 1) {
+  if (ss->ssl_ == NULL) return False();
+  if (SSL_shutdown(ss->ssl_) == 1) {
     return True();
   }
   return False();
@@ -587,8 +589,8 @@ Handle<Value> SecureStream::IsInitFinished(const Arguments& args) {
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
 
-  if (ss->pSSL == NULL) return False();
-  if (SSL_is_init_finished(ss->pSSL)) {
+  if (ss->ssl_ == NULL) return False();
+  if (SSL_is_init_finished(ss->ssl_)) {
     return True();
   }
   return False();
@@ -600,13 +602,13 @@ Handle<Value> SecureStream::VerifyPeer(const Arguments& args) {
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
 
-  if (ss->pSSL == NULL) return False();
-  if (!ss->shouldVerify) return False();
-  X509* peer_cert = SSL_get_peer_certificate(ss->pSSL);
+  if (ss->ssl_ == NULL) return False();
+  if (!ss->should_verify_) return False();
+  X509* peer_cert = SSL_get_peer_certificate(ss->ssl_);
   if (peer_cert==NULL) return False();
   X509_free(peer_cert);
 
-  long x509_verify_error = SSL_get_verify_result(ss->pSSL);
+  long x509_verify_error = SSL_get_verify_result(ss->ssl_);
 
   // Can also check for:
   // X509_V_ERR_CERT_HAS_EXPIRED
@@ -630,8 +632,8 @@ Handle<Value> SecureStream::GetCurrentCipher(const Arguments& args) {
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
   OPENSSL_CONST SSL_CIPHER *c;
 
-  if ( ss->pSSL == NULL ) return Undefined();
-  c = SSL_get_current_cipher(ss->pSSL);
+  if ( ss->ssl_ == NULL ) return Undefined();
+  c = SSL_get_current_cipher(ss->ssl_);
   if ( c == NULL ) return Undefined();
   Local<Object> info = Object::New();
   const char *cipher_name = SSL_CIPHER_get_name(c);
@@ -646,9 +648,9 @@ Handle<Value> SecureStream::Close(const Arguments& args) {
 
   SecureStream *ss = ObjectWrap::Unwrap<SecureStream>(args.Holder());
 
-  if (ss->pSSL != NULL) {
-    SSL_free(ss->pSSL);
-    ss->pSSL = NULL;
+  if (ss->ssl_ != NULL) {
+    SSL_free(ss->ssl_);
+    ss->ssl_ = NULL;
   }
   return True();
 }
@@ -692,7 +694,8 @@ void base64(unsigned char *input, int length, char** buf64, int* buf64_len) {
   BIO_set_flags(b64, BIO_FLAGS_BASE64_NO_NL);
   len = BIO_write(b64, input, length);
   assert(len == length);
-  BIO_flush(b64);
+  int r = BIO_flush(b64);
+  assert(r == 1);
   BIO_get_mem_ptr(b64, &bptr);
 
   *buf64_len = bptr->length;
@@ -803,7 +806,7 @@ int local_EVP_DecryptFinal_ex(EVP_CIPHER_CTX *ctx,
       return(0);
     }
 
-    if (b > (sizeof(ctx->final) / sizeof(ctx->final[0]))) {
+    if (b > (int)(sizeof(ctx->final) / sizeof(ctx->final[0]))) {
       EVPerr(EVP_F_EVP_DECRYPTFINAL,EVP_R_BAD_DECRYPT);
       return(0);
     }
@@ -960,6 +963,10 @@ class Cipher : public ObjectWrap {
 
     delete [] key_buf;
 
+    if (!r) {
+      return ThrowException(Exception::Error(String::New("CipherInit error")));
+    }
+
     return args.This();
   }
 
@@ -1004,6 +1011,10 @@ class Cipher : public ObjectWrap {
     delete [] key_buf;
     delete [] iv_buf;
 
+    if (!r) {
+      return ThrowException(Exception::Error(String::New("CipherInitIv error")));
+    }
+
     return args.This();
   }
 
@@ -1022,18 +1033,27 @@ class Cipher : public ObjectWrap {
     }
 
     unsigned char *out=0;
-    int out_len=0;
+    int out_len=0, r;
     if (Buffer::HasInstance(args[0])) {
-      Buffer *buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
-      int r = cipher->CipherUpdate(buffer->data(), buffer->length(), &out, &out_len);
+      Local<Object> buffer_obj = args[0]->ToObject();
+      char *buffer_data = Buffer::Data(buffer_obj);
+      size_t buffer_length = Buffer::Length(buffer_obj);
+
+      r = cipher->CipherUpdate(buffer_data, buffer_length, &out, &out_len);
     } else {
       char* buf = new char[len];
       ssize_t written = DecodeWrite(buf, len, args[0], enc);
       assert(written == len);
-      int r = cipher->CipherUpdate(buf, len,&out,&out_len);
+      r = cipher->CipherUpdate(buf, len,&out,&out_len);
       delete [] buf;
     }
 
+    if (!r) {
+      delete [] out;
+      Local<Value> exception = Exception::TypeError(String::New("DecipherUpdate fail"));
+      return ThrowException(exception);
+    }
+
     Local<Value> outString;
     if (out_len==0) {
       outString=String::New("");
@@ -1304,6 +1324,10 @@ class Decipher : public ObjectWrap {
 
     delete [] key_buf;
 
+    if (!r) {
+      return ThrowException(Exception::Error(String::New("DecipherInit error")));
+    }
+
     return args.This();
   }
 
@@ -1349,6 +1373,10 @@ class Decipher : public ObjectWrap {
     delete [] key_buf;
     delete [] iv_buf;
 
+    if (!r) {
+      return ThrowException(Exception::Error(String::New("DecipherInitIv error")));
+    }
+
     return args.This();
   }
 
@@ -1367,9 +1395,12 @@ class Decipher : public ObjectWrap {
     // if alloc_buf then buf must be deleted later
     bool alloc_buf = false;
     if (Buffer::HasInstance(args[0])) {
-      Buffer *buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
-      buf = buffer->data();
-      len = buffer->length();
+      Local<Object> buffer_obj = args[0]->ToObject();
+      char *buffer_data = Buffer::Data(buffer_obj);
+      size_t buffer_length = Buffer::Length(buffer_obj);
+
+      buf = buffer_data;
+      len = buffer_length;
     } else {
       alloc_buf = true;
       buf = new char[len];
@@ -1436,6 +1467,12 @@ class Decipher : public ObjectWrap {
     int out_len=0;
     int r = cipher->DecipherUpdate(buf, len, &out, &out_len);
 
+    if (!r) {
+      delete [] out;
+      Local<Value> exception = Exception::TypeError(String::New("DecipherUpdate fail"));
+      return ThrowException(exception);
+    }
+
     Local<Value> outString;
     if (out_len==0) {
       outString=String::New("");
@@ -1484,9 +1521,7 @@ class Decipher : public ObjectWrap {
 
     unsigned char* out_value;
     int out_len;
-    char* out_hexdigest;
-    int out_hex_len;
-    Local<Value> outString ;
+    Local<Value> outString;
 
     int r = cipher->DecipherFinal(&out_value, &out_len, false);
 
@@ -1529,8 +1564,6 @@ class Decipher : public ObjectWrap {
 
     unsigned char* out_value;
     int out_len;
-    char* out_hexdigest;
-    int out_hex_len;
     Local<Value> outString ;
 
     int r = cipher->DecipherFinal(&out_value, &out_len, true);
@@ -1670,6 +1703,10 @@ class Hmac : public ObjectWrap {
 
     delete [] buf;
 
+    if (!r) {
+      return ThrowException(Exception::Error(String::New("hmac error")));
+    }
+
     return args.This();
   }
 
@@ -1685,18 +1722,28 @@ class Hmac : public ObjectWrap {
       Local<Value> exception = Exception::TypeError(String::New("Bad argument"));
       return ThrowException(exception);
     }
+
+    int r;
 	
     if( Buffer::HasInstance(args[0])) {
-      Buffer *buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
-      int r = hmac->HmacUpdate(buffer->data(), buffer->length());
+      Local<Object> buffer_obj = args[0]->ToObject();
+      char *buffer_data = Buffer::Data(buffer_obj);
+      size_t buffer_length = Buffer::Length(buffer_obj);
+
+      r = hmac->HmacUpdate(buffer_data, buffer_length);
     } else {
       char* buf = new char[len];
       ssize_t written = DecodeWrite(buf, len, args[0], enc);
       assert(written == len);
-      int r = hmac->HmacUpdate(buf, len);
+      r = hmac->HmacUpdate(buf, len);
       delete [] buf;
     }
 
+    if (!r) {
+      Local<Value> exception = Exception::TypeError(String::New("HmacUpdate fail"));
+      return ThrowException(exception);
+    }
+
     return args.This();
   }
 
@@ -1832,18 +1879,26 @@ class Hash : public ObjectWrap {
       return ThrowException(exception);
     }
 
+    int r;
 
     if (Buffer::HasInstance(args[0])) {
-      Buffer *buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
-      int r = hash->HashUpdate(buffer->data(), buffer->length());
+      Local<Object> buffer_obj = args[0]->ToObject();
+      char *buffer_data = Buffer::Data(buffer_obj);
+      size_t buffer_length = Buffer::Length(buffer_obj);
+      r = hash->HashUpdate(buffer_data, buffer_length);
     } else {
       char* buf = new char[len];
       ssize_t written = DecodeWrite(buf, len, args[0], enc);
       assert(written == len);
-      int r = hash->HashUpdate(buf, len);
+      r = hash->HashUpdate(buf, len);
       delete[] buf;
     }
 
+    if (!r) {
+      Local<Value> exception = Exception::TypeError(String::New("HashUpdate fail"));
+      return ThrowException(exception);
+    }
+
     return args.This();
   }
 
@@ -1941,14 +1996,14 @@ class Sign : public ObjectWrap {
 
   int SignFinal(unsigned char** md_value,
                 unsigned int *md_len,
-                char* keyPem,
-                int keyPemLen) {
+                char* key_pem,
+                int key_pemLen) {
     if (!initialised_) return 0;
 
     BIO *bp = NULL;
     EVP_PKEY* pkey;
     bp = BIO_new(BIO_s_mem());
-    if(!BIO_write(bp, keyPem, keyPemLen)) return 0;
+    if(!BIO_write(bp, key_pem, key_pemLen)) return 0;
 
     pkey = PEM_read_bio_PrivateKey( bp, NULL, NULL, NULL );
     if (pkey == NULL) return 0;
@@ -1987,6 +2042,10 @@ class Sign : public ObjectWrap {
 
     bool r = sign->SignInit(*signType);
 
+    if (!r) {
+      return ThrowException(Exception::Error(String::New("SignInit error")));
+    }
+
     return args.This();
   }
 
@@ -2003,17 +2062,27 @@ class Sign : public ObjectWrap {
       return ThrowException(exception);
     }
 
+    int r;
+
     if (Buffer::HasInstance(args[0])) {
-      Buffer *buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
-      int r = sign->SignUpdate(buffer->data(), buffer->length());
+      Local<Object> buffer_obj = args[0]->ToObject();
+      char *buffer_data = Buffer::Data(buffer_obj);
+      size_t buffer_length = Buffer::Length(buffer_obj);
+
+      r = sign->SignUpdate(buffer_data, buffer_length);
     } else {
       char* buf = new char[len];
       ssize_t written = DecodeWrite(buf, len, args[0], enc);
       assert(written == len);
-      int r = sign->SignUpdate(buf, len);
+      r = sign->SignUpdate(buf, len);
       delete [] buf;
     }
 
+    if (!r) {
+      Local<Value> exception = Exception::TypeError(String::New("SignUpdate fail"));
+      return ThrowException(exception);
+    }
+
     return args.This();
   }
 
@@ -2129,7 +2198,7 @@ class Verify : public ObjectWrap {
   }
 
 
-  int VerifyFinal(char* keyPem, int keyPemLen, unsigned char* sig, int siglen) {
+  int VerifyFinal(char* key_pem, int key_pemLen, unsigned char* sig, int siglen) {
     if (!initialised_) return 0;
 
     BIO *bp = NULL;
@@ -2137,7 +2206,7 @@ class Verify : public ObjectWrap {
     X509 *x509;
 
     bp = BIO_new(BIO_s_mem());
-    if(!BIO_write(bp, keyPem, keyPemLen)) return 0;
+    if(!BIO_write(bp, key_pem, key_pemLen)) return 0;
 
     x509 = PEM_read_bio_X509(bp, NULL, NULL, NULL );
     if (x509==NULL) return 0;
@@ -2185,6 +2254,10 @@ class Verify : public ObjectWrap {
 
     bool r = verify->VerifyInit(*verifyType);
 
+    if (!r) {
+      return ThrowException(Exception::Error(String::New("VerifyInit error")));
+    }
+
     return args.This();
   }
 
@@ -2202,17 +2275,27 @@ class Verify : public ObjectWrap {
       return ThrowException(exception);
     }
 
+    int r;
+
     if(Buffer::HasInstance(args[0])) {
-      Buffer *buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
-      int r = verify->VerifyUpdate(buffer->data(), buffer->length());
+      Local<Object> buffer_obj = args[0]->ToObject();
+      char *buffer_data = Buffer::Data(buffer_obj);
+      size_t buffer_length = Buffer::Length(buffer_obj);
+
+      r = verify->VerifyUpdate(buffer_data, buffer_length);
     } else {
       char* buf = new char[len];
       ssize_t written = DecodeWrite(buf, len, args[0], enc);
       assert(written == len);
-      int r = verify->VerifyUpdate(buf, len);
+      r = verify->VerifyUpdate(buf, len);
       delete [] buf;
     }
 
+    if (!r) {
+      Local<Value> exception = Exception::TypeError(String::New("VerifyUpdate fail"));
+      return ThrowException(exception);
+    }
+
     return args.This();
   }
 
