@@ -21,8 +21,8 @@ class SecureContext : ObjectWrap {
  public:
   static void Initialize(v8::Handle<v8::Object> target);
 
-  SSL_CTX *pCtx;
-  X509_STORE *caStore;
+  SSL_CTX *ctx_;
+  X509_STORE *ca_store_;
 
  protected:
   static v8::Handle<v8::Value> New(const v8::Arguments& args);
@@ -34,6 +34,8 @@ class SecureContext : ObjectWrap {
   static v8::Handle<v8::Value> Close(const v8::Arguments& args);
 
   SecureContext() : ObjectWrap() {
+    ctx_ = NULL;
+    ca_store_ = NULL;
   }
 
   ~SecureContext() {
@@ -49,11 +51,12 @@ class SecureStream : ObjectWrap {
 
  protected:
   static v8::Handle<v8::Value> New(const v8::Arguments& args);
-  static v8::Handle<v8::Value> ReadInject(const v8::Arguments& args);
-  static v8::Handle<v8::Value> ReadExtract(const v8::Arguments& args);
-  static v8::Handle<v8::Value> WriteCanExtract(const v8::Arguments& args);
-  static v8::Handle<v8::Value> WriteExtract(const v8::Arguments& args);
-  static v8::Handle<v8::Value> WriteInject(const v8::Arguments& args);
+  static v8::Handle<v8::Value> EncIn(const v8::Arguments& args);
+  static v8::Handle<v8::Value> ClearOut(const v8::Arguments& args);
+  static v8::Handle<v8::Value> ClearPending(const v8::Arguments& args);
+  static v8::Handle<v8::Value> EncPending(const v8::Arguments& args);
+  static v8::Handle<v8::Value> EncOut(const v8::Arguments& args);
+  static v8::Handle<v8::Value> ClearIn(const v8::Arguments& args);
   static v8::Handle<v8::Value> GetPeerCertificate(const v8::Arguments& args);
   static v8::Handle<v8::Value> IsInitFinished(const v8::Arguments& args);
   static v8::Handle<v8::Value> VerifyPeer(const v8::Arguments& args);
@@ -62,16 +65,23 @@ class SecureStream : ObjectWrap {
   static v8::Handle<v8::Value> Close(const v8::Arguments& args);
 
   SecureStream() : ObjectWrap() {
+    bio_read_ = bio_write_ = NULL;
+    ssl_ = NULL;
   }
 
   ~SecureStream() {
+    if (ssl_ != NULL) {
+      SSL_free(ssl_);
+      ssl_ = NULL;
+    }
   }
 
  private:
-  BIO *pbioRead;
-  BIO *pbioWrite;
-  SSL *pSSL;
-  bool server;
+  BIO *bio_read_;
+  BIO *bio_write_;
+  SSL *ssl_;
+  bool is_server_; /* coverity[member_decl] */
+  bool should_verify_; /* coverity[member_decl] */
 };
 
 void InitCrypto(v8::Handle<v8::Object> target);
