@@ -7,6 +7,8 @@
 #include <http_parser.h>
 
 #include <strings.h>  /* strcasecmp() */
+#include <string.h>  /* strdup() */
+#include <stdlib.h>  /* free() */
 
 // This is a binding to http_parser (http://github.com/ry/http-parser)
 // The goal is to decouple sockets from parsing for more javascript-level
@@ -19,7 +21,7 @@
 // No copying is performed when slicing the buffer, only small reference
 // allocations.
 
- 
+
 namespace node {
 
 using namespace v8;
@@ -50,6 +52,10 @@ static Persistent<String> move_sym;
 static Persistent<String> propfind_sym;
 static Persistent<String> proppatch_sym;
 static Persistent<String> unlock_sym;
+static Persistent<String> report_sym;
+static Persistent<String> mkactivity_sym;
+static Persistent<String> checkout_sym;
+static Persistent<String> merge_sym;
 static Persistent<String> unknown_method_sym;
 
 static Persistent<String> method_sym;
@@ -62,6 +68,15 @@ static Persistent<String> upgrade_sym;
 
 static struct http_parser_settings settings;
 
+
+// This is a hack to get the current_buffer to the callbacks with the least
+// amount of overhead. Nothing else will run while http_parser_execute()
+// runs, therefore this pointer can be set and used for the execution.
+static Local<Value>* current_buffer;
+static char* current_buffer_data;
+static size_t current_buffer_len;
+
+
 // Callback prototype for http_cb
 #define DEFINE_HTTP_CB(name)                                             \
   static int name(http_parser *p) {                                      \
@@ -82,16 +97,16 @@ static struct http_parser_settings settings;
 #define DEFINE_HTTP_DATA_CB(name)                                        \
   static int name(http_parser *p, const char *at, size_t length) {       \
     Parser *parser = static_cast<Parser*>(p->data);                      \
-    assert(parser->buffer_);                                             \
+    assert(current_buffer);                                              \
     Local<Value> cb_value = parser->handle_->Get(name##_sym);            \
     if (!cb_value->IsFunction()) return 0;                               \
     Local<Function> cb = Local<Function>::Cast(cb_value);                \
-    Local<Value> argv[3] = { Local<Value>::New(parser->buffer_->handle_) \
-                           , Integer::New(at - parser->buffer_->data())  \
+    Local<Value> argv[3] = { *current_buffer                             \
+                           , Integer::New(at - current_buffer_data)      \
                            , Integer::New(length)                        \
                            };                                            \
     Local<Value> ret = cb->Call(parser->handle_, 3, argv);               \
-    assert(parser->buffer_);                                             \
+    assert(current_buffer);                                              \
     if (ret.IsEmpty()) {                                                 \
       parser->got_exception_ = true;                                     \
       return -1;                                                         \
@@ -119,6 +134,10 @@ method_to_str(unsigned short m) {
     case HTTP_PROPFIND:   return propfind_sym;
     case HTTP_PROPPATCH:  return proppatch_sym;
     case HTTP_UNLOCK:     return unlock_sym;
+    case HTTP_REPORT:     return report_sym;
+    case HTTP_MKACTIVITY: return mkactivity_sym;
+    case HTTP_CHECKOUT:   return checkout_sym;
+    case HTTP_MERGE:      return merge_sym;
     default:              return unknown_method_sym;
   }
 }
@@ -127,12 +146,10 @@ method_to_str(unsigned short m) {
 class Parser : public ObjectWrap {
  public:
   Parser(enum http_parser_type type) : ObjectWrap() {
-    buffer_ = NULL;
     Init(type);
   }
 
   ~Parser() {
-    assert(buffer_ == NULL && "Destroying a parser while it's parsing");
   }
 
   DEFINE_HTTP_CB(on_message_begin)
@@ -192,7 +209,7 @@ class Parser : public ObjectWrap {
 
     String::Utf8Value type(args[0]->ToString());
 
-    Parser *parser; 
+    Parser *parser;
 
     if (0 == strcasecmp(*type, "request")) {
       parser = new Parser(HTTP_REQUEST);
@@ -204,7 +221,7 @@ class Parser : public ObjectWrap {
     }
 
     parser->Wrap(args.This());
-    assert(!parser->buffer_);
+    assert(!current_buffer);
 
     return args.This();
   }
@@ -215,47 +232,56 @@ class Parser : public ObjectWrap {
 
     Parser *parser = ObjectWrap::Unwrap<Parser>(args.This());
 
-    assert(!parser->buffer_);
-    if (parser->buffer_) {
+    assert(!current_buffer);
+    assert(!current_buffer_data);
+
+    if (current_buffer) {
       return ThrowException(Exception::TypeError(
             String::New("Already parsing a buffer")));
     }
 
-    if (!Buffer::HasInstance(args[0])) {
+    Local<Value> buffer_v = args[0];
+
+    if (!Buffer::HasInstance(buffer_v)) {
       return ThrowException(Exception::TypeError(
             String::New("Argument should be a buffer")));
     }
 
-    Buffer * buffer = ObjectWrap::Unwrap<Buffer>(args[0]->ToObject());
+    Local<Object> buffer_obj = buffer_v->ToObject();
+    char *buffer_data = Buffer::Data(buffer_obj);
+    size_t buffer_len = Buffer::Length(buffer_obj);
 
     size_t off = args[1]->Int32Value();
-    if (off >= buffer->length()) {
+    if (off >= buffer_len) {
       return ThrowException(Exception::Error(
             String::New("Offset is out of bounds")));
     }
 
     size_t len = args[2]->Int32Value();
-    if (off+len > buffer->length()) {
+    if (off+len > buffer_len) {
       return ThrowException(Exception::Error(
             String::New("Length is extends beyond buffer")));
     }
 
     // Assign 'buffer_' while we parse. The callbacks will access that varible.
-    parser->buffer_ = buffer;
+    current_buffer = &buffer_v;
+    current_buffer_data = buffer_data;
+    current_buffer_len = buffer_len;
     parser->got_exception_ = false;
 
     size_t nparsed =
-      http_parser_execute(&parser->parser_, &settings, buffer->data()+off, len);
+      http_parser_execute(&parser->parser_, &settings, buffer_data + off, len);
 
     // Unassign the 'buffer_' variable
-    assert(parser->buffer_);
-    parser->buffer_ = NULL;
+    assert(current_buffer);
+    current_buffer = NULL;
+    current_buffer_data = NULL;
 
     // If there was an exception in one of the callbacks
     if (parser->got_exception_) return Local<Value>();
 
     Local<Integer> nparsed_obj = Integer::New(nparsed);
-    // If there was a parse error in one of the callbacks 
+    // If there was a parse error in one of the callbacks
     // TODO What if there is an error on EOF?
     if (!parser->parser_.upgrade && nparsed != len) {
       Local<Value> e = Exception::Error(String::NewSymbol("Parse Error"));
@@ -272,7 +298,7 @@ class Parser : public ObjectWrap {
 
     Parser *parser = ObjectWrap::Unwrap<Parser>(args.This());
 
-    assert(!parser->buffer_);
+    assert(!current_buffer);
     parser->got_exception_ = false;
 
     http_parser_execute(&(parser->parser_), &settings, NULL, 0);
@@ -303,18 +329,95 @@ class Parser : public ObjectWrap {
  private:
 
   void Init (enum http_parser_type type) {
-    assert(buffer_ == NULL); // don't call this during Execute()
     http_parser_init(&parser_, type);
-
     parser_.data = this;
   }
 
-  Buffer * buffer_;  // The buffer currently being parsed.
   bool got_exception_;
   http_parser parser_;
 };
 
 
+static Handle<Value> UrlDecode (const Arguments& args) {
+  HandleScope scope;
+
+  if (!args[0]->IsString()) {
+    return ThrowException(Exception::TypeError(
+          String::New("First arg must be a string")));
+  }
+
+  bool decode_spaces = args[1]->IsTrue();
+
+  String::Utf8Value in_v(args[0]->ToString());
+  size_t l = in_v.length();
+  char* out = strdup(*in_v);
+
+  enum { CHAR, HEX0, HEX1 } state = CHAR;
+
+  int n, m, hexchar;
+  size_t in_index = 0, out_index = 0;
+  char c;
+  for (; in_index <= l; in_index++) {
+    c = out[in_index];
+    switch (state) {
+      case CHAR:
+        switch (c) {
+          case '%':
+            n = 0;
+            m = 0;
+            state = HEX0;
+            break;
+          case '+':
+            if (decode_spaces) c = ' ';
+            // pass thru
+          default:
+            out[out_index++] = c;
+            break;
+        }
+        break;
+
+      case HEX0:
+        state = HEX1;
+        hexchar = c;
+        if ('0' <= c && c <= '9') {
+          n = c - '0';
+        } else if ('a' <= c && c <= 'f') {
+          n = c - 'a' + 10;
+        } else if ('A' <= c && c <= 'F') {
+          n = c - 'A' + 10;
+        } else {
+          out[out_index++] = '%';
+          out[out_index++] = c;
+          state = CHAR;
+          break;
+        }
+        break;
+
+      case HEX1:
+        state = CHAR;
+        if ('0' <= c && c <= '9') {
+          m = c - '0';
+        } else if ('a' <= c && c <= 'f') {
+          m = c - 'a' + 10;
+        } else if ('A' <= c && c <= 'F') {
+          m = c - 'A' + 10;
+        } else {
+          out[out_index++] = '%';
+          out[out_index++] = hexchar;
+          out[out_index++] = c;
+          break;
+        }
+        out[out_index++] = 16*n + m;
+        break;
+    }
+  }
+
+  Local<String> out_v = String::New(out, out_index-1);
+  free(out);
+  return scope.Close(out_v);
+}
+
+
 void InitHttpParser(Handle<Object> target) {
   HandleScope scope;
 
@@ -327,6 +430,7 @@ void InitHttpParser(Handle<Object> target) {
   NODE_SET_PROTOTYPE_METHOD(t, "reinitialize", Parser::Reinitialize);
 
   target->Set(String::NewSymbol("HTTPParser"), t->GetFunction());
+  NODE_SET_METHOD(target, "urlDecode", UrlDecode);
 
   on_message_begin_sym    = NODE_PSYMBOL("onMessageBegin");
   on_path_sym             = NODE_PSYMBOL("onPath");
@@ -354,6 +458,10 @@ void InitHttpParser(Handle<Object> target) {
   propfind_sym = NODE_PSYMBOL("PROPFIND");
   proppatch_sym = NODE_PSYMBOL("PROPPATCH");
   unlock_sym = NODE_PSYMBOL("UNLOCK");
+  report_sym = NODE_PSYMBOL("REPORT");
+  mkactivity_sym = NODE_PSYMBOL("MKACTIVITY");
+  checkout_sym = NODE_PSYMBOL("CHECKOUT");
+  merge_sym = NODE_PSYMBOL("MERGE");
   unknown_method_sym = NODE_PSYMBOL("UNKNOWN_METHOD");
 
   method_sym = NODE_PSYMBOL("method");
@@ -378,3 +486,4 @@ void InitHttpParser(Handle<Object> target) {
 
 }  // namespace node
 
+NODE_MODULE(node_http_parser, node::InitHttpParser);
