@@ -5,17 +5,24 @@
 #define NODE_VERSION_H
 
 #define NODE_MAJOR_VERSION 0
-#define NODE_MINOR_VERSION 2
+#define NODE_MINOR_VERSION 3
 #define NODE_PATCH_VERSION 0
+#define NODE_VERSION_IS_RELEASE 1
 
 #ifndef NODE_STRINGIFY
 #define NODE_STRINGIFY(n) NODE_STRINGIFY_HELPER(n)
 #define NODE_STRINGIFY_HELPER(n) #n
 #endif
 
-#define NODE_VERSION_STRING   NODE_STRINGIFY(NODE_MAJOR_VERSION) "." \
+#if NODE_VERSION_IS_RELEASE
+# define NODE_VERSION_STRING  NODE_STRINGIFY(NODE_MAJOR_VERSION) "." \
                               NODE_STRINGIFY(NODE_MINOR_VERSION) "." \
                               NODE_STRINGIFY(NODE_PATCH_VERSION)
+#else
+# define NODE_VERSION_STRING  NODE_STRINGIFY(NODE_MAJOR_VERSION) "." \
+                              NODE_STRINGIFY(NODE_MINOR_VERSION) "." \
+                              NODE_STRINGIFY(NODE_PATCH_VERSION) "-pre"
+#endif
 
 #define NODE_VERSION "v" NODE_VERSION_STRING
 
