@@ -1,5 +1,5 @@
 common = require("../common");
-assert = common.assert
+assert = common.assert;
 
 http = require("http");
 
@@ -21,10 +21,10 @@ server.addListener('listening', function () {
    http.cat('http://localhost:'+common.PORT+'/', 'utf8', function (err, content) {
      requests_complete++;
      if (err) {
-       print("-");
+       common.print("-");
      } else {
-       assert.equal(body, content)
-       print(".");
+       assert.equal(body, content);
+       common.print(".");
        requests_ok++;
      }
      if (requests_complete == request_count) {
