@@ -1,7 +1,7 @@
 common = require("../common");
-assert = common.assert
+assert = common.assert;
 
-var sys=require('sys');
+var util=require('util');
 var net=require('net');
 var fs=require('fs');
 var crypto=require('crypto');
@@ -9,7 +9,12 @@ var crypto=require('crypto');
 var keyPem = fs.readFileSync(common.fixturesDir + "/cert.pem");
 var certPem = fs.readFileSync(common.fixturesDir + "/cert.pem");
 
-var credentials = crypto.createCredentials({key:keyPem, cert:certPem});
+try{
+  var credentials = crypto.createCredentials({key:keyPem, cert:certPem});
+} catch (e) {
+  console.log("Not compiled with OPENSSL support.");
+  process.exit();
+}
 var i = 0;
 var server = net.createServer(function (connection) {
   connection.setSecure(credentials);
