@@ -1,6 +1,6 @@
 var c = require("./b/c");
 
-debug("load fixtures/a.js");
+common.debug("load fixtures/a.js");
 
 var string = "A";
 
@@ -18,6 +18,8 @@ exports.D = function () {
   return c.D();
 };
 
+exports.number = 42;
+
 process.addListener("exit", function () {
   string = "A done";
 });
