@@ -1,9 +1,8 @@
-var exec = require('child_process').exec, 
-    puts = require('sys').puts;
+var exec = require('child_process').exec;
 
 [0, 1].forEach(function(i) {
  exec('ls', function(err, stdout, stderr) {
-   puts(i);
+   console.log(i);
    throw new Error('hello world');
  });
 });
