@@ -3,7 +3,7 @@ assert = common.assert
 var path = require('path');
 var Buffer = require('buffer').Buffer;
 var fs = require('fs');
-var fn = path.join(common.fixturesDir, "write.txt");
+var fn = path.join(common.tmpDir, "write.txt");
 var expected = "ümlaut.";
 var found;
 
@@ -11,7 +11,7 @@ fs.open(fn, 'w', 0644, function (err, fd) {
   if (err) throw err;
   console.log('open done');
   fs.write(fd, '', 0, 'utf8', function(err, written) {
-      assert.fail('zero length write should not go through to callback');
+    assert.equal(0, written);
   });
   fs.write(fd, expected, 0, "utf8", function (err, written) {
     console.log('write done');
