@@ -1,6 +1,7 @@
 common = require("../common");
 assert = common.assert
-var path = require('path');
+var path = require('path'),
+    fs   = require('fs');
 
 common.debug("load test-module-loading.js");
 
@@ -15,25 +16,25 @@ var d4 = require("../fixtures/b/d");
 
 assert.equal(false, false, "testing the test program.");
 
-assert.equal(true, a.A instanceof Function);
+assert.equal(true, common.indirectInstanceOf(a.A, Function));
 assert.equal("A", a.A());
 
-assert.equal(true, a.C instanceof Function);
+assert.equal(true, common.indirectInstanceOf(a.C, Function));
 assert.equal("C", a.C());
 
-assert.equal(true, a.D instanceof Function);
+assert.equal(true, common.indirectInstanceOf(a.D, Function));
 assert.equal("D", a.D());
 
-assert.equal(true, d.D instanceof Function);
+assert.equal(true, common.indirectInstanceOf(d.D, Function));
 assert.equal("D", d.D());
 
-assert.equal(true, d2.D instanceof Function);
+assert.equal(true, common.indirectInstanceOf(d2.D, Function));
 assert.equal("D", d2.D());
 
-assert.equal(true, d3.D instanceof Function);
+assert.equal(true, common.indirectInstanceOf(d3.D, Function));
 assert.equal("D", d3.D());
 
-assert.equal(true, d4.D instanceof Function);
+assert.equal(true, common.indirectInstanceOf(d4.D, Function));
 assert.equal("D", d4.D());
 
 assert.ok((new a.SomeClass) instanceof c.SomeClass);
@@ -52,7 +53,7 @@ assert.equal(root.sayHello(), root.hello);
 common.debug("test name clashes");
 // this one exists and should import the local module
 var my_path = require("./path");
-assert.equal(true, my_path.path_func instanceof Function);
+assert.ok(common.indirectInstanceOf(my_path.path_func, Function));
 // this one does not exist and should throw
 assert.throws(function() { require("./utils")});
 
@@ -64,41 +65,29 @@ try {
   assert.equal("blah", e.message);
 }
 
-var errorThrownAsync = false;
-require.async("../fixtures/throws_error1", function(err, a) {
-  if (err) {
-    errorThrownAsync = true;
-    assert.equal("blah", err.message);
-  }
-});
-
 assert.equal(require('path').dirname(__filename), __dirname);
 
-var asyncRun = false;
-require.async('../fixtures/a1', function (err, a) {
-  if (err) throw err;
-  assert.equal("A", a.A());
-  asyncRun = true;
-});
-
-common.debug('load custom file types with registerExtension');
-require.registerExtension('.test', function(content) {
+common.debug('load custom file types with extensions');
+require.extensions['.test'] = function (module, filename) {
+  var content = fs.readFileSync(filename).toString();
   assert.equal("this is custom source\n", content);
-
-  return content.replace("this is custom source", "exports.test = 'passed'");
-});
+  content = content.replace("this is custom source", "exports.test = 'passed'");
+  module._compile(content, filename);
+};
 
 assert.equal(require('../fixtures/registerExt').test, "passed");
+// unknown extension, load as .js
+assert.equal(require('../fixtures/registerExt.hello.world').test, "passed");
 
 common.debug('load custom file types that return non-strings');
-require.registerExtension('.test', function(content) {
-  return {
+require.extensions['.test'] = function (module, filename) {
+  module.exports = {
     custom: 'passed'
   };
-});
+};
 
 assert.equal(require('../fixtures/registerExt2').custom, 'passed');
-debug("load modules by absolute id, then change require.paths, and load another module with the same absolute id.");
+common.debug("load modules by absolute id, then change require.paths, and load another module with the same absolute id.");
 // this will throw if it fails.
 var foo = require("../fixtures/require-path/p1/foo");
 process.assert(foo.bar.expect === foo.bar.actual);
@@ -113,24 +102,14 @@ try {
   assert.equal(err.message, "Cannot find module '../fixtures/empty'");
 }
 
-var asyncRequireDir = false;
-require.async("../fixtures/empty", function (err, a) {
-  assert.ok(err);
-
-  if (err) {
-    asyncRequireDir = true;
-    assert.equal(err.message, "Cannot find module '../fixtures/empty'");
-  }
-});
-
 // Check load order is as expected
 common.debug('load order');
 
 var loadOrder = '../fixtures/module-load-order/',
     msg       = "Load order incorrect.";
 
-require.registerExtension('.reg',  function(content) { return content; });
-require.registerExtension('.reg2', function(content) { return content; });
+require.extensions['.reg'] = require.extensions['.js'];
+require.extensions['.reg2'] = require.extensions['.js'];
 
 assert.equal(require(loadOrder + 'file1').file1, 'file1',            msg);
 assert.equal(require(loadOrder + 'file2').file2, 'file2.js',         msg);
@@ -152,28 +131,22 @@ assert.equal(require(loadOrder + 'file8').file8, 'file8/index.reg',  msg);
 assert.equal(require(loadOrder + 'file9').file9, 'file9/index.reg2', msg);
 
 process.addListener("exit", function () {
-  assert.equal(true, a.A instanceof Function);
+  assert.ok(common.indirectInstanceOf(a.A, Function));
   assert.equal("A done", a.A());
 
-  assert.equal(true, a.C instanceof Function);
+  assert.ok(common.indirectInstanceOf(a.C, Function));
   assert.equal("C done", a.C());
 
-  assert.equal(true, a.D instanceof Function);
+  assert.ok(common.indirectInstanceOf(a.D, Function));
   assert.equal("D done", a.D());
 
-  assert.equal(true, d.D instanceof Function);
+  assert.ok(common.indirectInstanceOf(d.D, Function));
   assert.equal("D done", d.D());
 
-  assert.equal(true, d2.D instanceof Function);
+  assert.ok(common.indirectInstanceOf(d2.D, Function));
   assert.equal("D done", d2.D());
 
   assert.equal(true, errorThrown);
 
-  assert.equal(true, asyncRun);
-
-  assert.equal(true, errorThrownAsync);
-
-  assert.equal(true, asyncRequireDir);
-
   console.log("exit");
 });
