@@ -1,7 +1,6 @@
 common = require("../common");
 assert = common.assert
 var fs = require('fs');
-var sys = require('sys');
 var net = require('net');
 
 var have_openssl;
@@ -18,7 +17,12 @@ var caPem = fs.readFileSync(common.fixturesDir+"/test_ca.pem", 'ascii');
 var certPem = fs.readFileSync(common.fixturesDir+"/test_cert.pem", 'ascii');
 var keyPem = fs.readFileSync(common.fixturesDir+"/test_key.pem", 'ascii');
 
-var credentials = crypto.createCredentials({key:keyPem, cert:certPem, ca:caPem});
+try{
+  var credentials = crypto.createCredentials({key:keyPem, cert:certPem, ca:caPem});
+} catch (e) {
+  console.log("Not compiled with OPENSSL support.");
+  process.exit();
+}
 
 var testData = "TEST123";
 var serverData = '';
@@ -41,7 +45,8 @@ var secureServer = net.createServer(function (connection) {
          + '"issuer":"/C=UK/ST=Acknack Ltd/L=Rhys Jones/O=node.js'
          + '/OU=Test TLS Certificate/CN=localhost","valid_from":'
          + '"Nov 11 09:52:22 2009 GMT","valid_to":'
-         + '"Nov  6 09:52:22 2029 GMT"}');
+         + '"Nov  6 09:52:22 2029 GMT",'
+         + '"fingerprint":"2A:7A:C2:DD:E5:F9:CC:53:72:35:99:7A:02:5A:71:38:52:EC:8A:DF"}');
 
   });
 
@@ -76,7 +81,8 @@ secureServer.addListener("listening", function() {
       + '"issuer":"/C=UK/ST=Acknack Ltd/L=Rhys Jones/O=node.js'
       + '/OU=Test TLS Certificate/CN=localhost","valid_from":'
       + '"Nov 11 09:52:22 2009 GMT","valid_to":'
-      + '"Nov  6 09:52:22 2029 GMT"}');
+      + '"Nov  6 09:52:22 2029 GMT",'
+      + '"fingerprint":"2A:7A:C2:DD:E5:F9:CC:53:72:35:99:7A:02:5A:71:38:52:EC:8A:DF"}');
 
     secureClient.write(testData);
     secureClient.end();
@@ -94,4 +100,4 @@ secureServer.addListener("listening", function() {
 process.addListener("exit", function () {
   assert.ok(gotSecureServer, "Did not get secure event for server");
   assert.ok(gotSecureClient, "Did not get secure event for clientr");
-});
\ No newline at end of file
+});
