@@ -34,7 +34,6 @@ var fs = require('fs');
 var net = require('net');
 var netBinding = process.binding('net');
 var path = require('path');
-var sys = require('sys');
 
 var DATA = {
   'ppid' : process.pid,
@@ -54,7 +53,7 @@ var logChild = function(d) {
 
   d.split('\n').forEach(function(l) {
     if (l.length > 0) {
-      sys.common.debug('CHILD: ' + l);
+      common.debug('CHILD: ' + l);
     }
   });
 };
